.TH routesim l "April 1999"

.SH NAME

routesim - Dynamic routing simulator

.SH SYNOPSIS

.B routesim
[options]

.B routesim
.B
-conf
.I
configuration file

.SH AVAILABILITY

Compiled for Solaris (sparc), IRIX64, and Linux (x86) platforms.


.SH DESCRIPTION
.I routesim
is an event-driven simulator that facilitates the study of
source-directed link-state dynamic and QoS routing in large networks.
To reduce simulation overhead,
.I routesim
operates at the flow (or connection) level, rather than simulating
individual data packets.

The simulator takes as input a variety of parameters that allow
precise control over network traffic characteristics, network
topology, routing algorithm and policies, and link-state update
policy.  Though all parameters can be specified on the command line,
it is more convenient to place them in a
.I
configuration file.
Parameters specified on the command line override any specification in
the
.I
configuration file.

.SH OPTIONS
Each option is shown with both its short and long forms.  The short
form is used on the command line and the long form, shown in
parentheses, is used in the configuration file.  Most options have
arguments that are either values (i.e., numbers) or strings.  When the
argument is listed as
.I
value
it should be a number.  Arguments shown as [<opt1> | <opt2> ...] require
one of the given <opt> values.

.P
.TP 15
.B \-accuracy [CURRENT | STALE]
(link-state-accuracy)
.br
CURRENT causes link-state to be perfectly accurate, while STALE makes
link-state subject to updates.

.TP
.B \-alpha
.I
value

(link-cost-alpha)
.br
Link cost function exponent (steepness)

.TP
.B \-altrouting [ROUND_ROBIN | STICKY | NO_ALT]
(alternate-routing)
.br
Alternate routing policy when using precomputed routing and depth-first
route extraction.  No effect with on-demand routing.

.TP
.B \-arrdist [WEIBULL | POISSON]
(arrival-dist)
.br
Flow interarrival distribution to use.

.TP
.B \-arrscale
.I
value

(arrival-scale)
.br
Scale parameter for interarrival distribution.  In the case of
POISSON, scale is the mean.

.TP
.B \-arrshape
.I
value

(arrival-shape)
.br
Shape parameter for WEIBULL flow interarrival distribution.

.TP
.B \-arrspec [ARR_UNIFORM | ARR_FILE]
(arrival-spec)
.br
Traffic matrix specification.  ARR_UNIFORM gives a uniform traffic
pattern, while ARR_FILE reads the src-dest arrival rates from a file
called
.I
rstraffic.dat

.TP
.B \-bandwidth
.I
value

(mean-bandwidth)
.br
Mean requested flow bandwidth.

.TP
.B \-blocking [NON_BLOCKING | BLOCKING]
(blocking-policy)
.br
Blocking model to use in the case of SINGLE network-model option.  See
the netmodel option.

.TP
.B \-bwclass
.I
value

(bw-class-size)
.br
Base bandwidth class size, B (for equal class based updates).  Avail
bw will be advertised as ..., (-B,0), (0,B), (B,2B), (2B,3B), ...

.TP
.B \-costconst
.I
value

(link-cost-constant)
.br
Additive bias for link costs when using SPT routing-algorithm option.

.TP
.B \-costlevels
.I
value

(link-cost-levels)
.br
Number of discrete cost levels when using SPT routing-algorithm option.

.TP
.B \-dist [EXP_SINGLE_COST | EXP_MINHOP_BIAS | HOPCOUNT
.B\ | WIDEST]

(distance-function)
.br
Cost function for SPT routing-algorithm.  HOPCOUNT produces
shortest-path routes.  EXP_MINHOP_BIAS uses the exponential cost
function but always biases toward shorter routes.  EXP_SINGLE_COST
uses the exponential cost function but adds a constant defined by the
link-cost-constant option.  WIDEST produces a shortest-widest
algorithm with ties between multiple min-hop paths broken by choosing
the one with most available link bandwidth.

.TP
.B \-dualalg [SPT | WIDE_SHORT]

(dual-dyn-algorithm)
.br
Dynamic routing algorithm to use in DUAL network model for hybrid
routing.  SPT uses the Dijkstra-based algorithms with policies
specified by the normal SPT options (e.g., distance-function,
alternate-routing, multi-route-policy, prune-policy).  WIDE_SHORT uses
the Bellman-Ford-based widest-shortest path algorithm.

.TP
.B \-dumptop
.br
(dump-topology)
.br
Dump the topology to a file
.I
rstopology.dat.out

.TP
.B \-failrecomp
.br
(recompute-after-fail)
.br
In precomputed routing, recompute routes after final set-up failure

.TP
.B \-feascheck
.br
(feasibility-check)
.br
In precomputed routing, perform feasibility check on extraction.

.TP
.B \-flowtrigger
.I
value

(flow-trigger)
.br
In DUAL nework-model, this is the long-lived flow detection threshold (time)

.TP
.B \-holdspec [EXP | PARETO | USERDEF]
(holding-time-spec)
.br
Flow duration distribution, exponential, Pareto, or user defined from
a file
.I
holdcdf.dat

.TP
.B \-hopthresh
.I
value

(hopcount-threshold)
.br
Number of hops allowed beyond min-hop route.  Only applies in
WIDE_SHORT routing-algorithm option.

.TP
.B \-hscale
.I
value

(holding-time-scale)
.br
Scale parameter for Pareto duration distribution or mean for
exponential distribution.

.TP
.B \-hshape
.I
value

(holding-time-shape)
.br
Shape parameter for Pareto duration distribution.

.TP
.B \-kndim
.I
value

(kn-dimen)
.br
Dimensions in k-ary n-cube.  Only applies in kncube topology-type option.

.TP
.B \-knedge
.I
value

(kn-edge)
.br
Edges in k-ary n-cube.  Only applies in kncube topology-type option.

.TP
.B \-level
.I
value

(confidence-level)
.br
Confidence level on blocking rate or overutiliziation, depending on
blocking model.

.TP
.B \-mail
.I
address

(mail-recipient)
.br
Recipient for simulation notifications.

.TP
.B \-maxdiam
.I
value

(max-rand-diameter)
.br
Maximum diameter for random topology generated at simulation time.

.TP
.B \-maxhops
.I
value

(max-hopcount)
.br
Maximum hopcount allowed.  Checked during route extraction phase.

.TP
.B \-maxsigs
.I
value

(max-sig-attempts)
.br
Maximum number of signaling attempts per flow request -- applicable
for precomputed routing.

.TP
.B \-mincost
.I
value

(link-cost-minval)
.br
Available bandwdith corresponding to minimum link cost value, when
using SPT routing-algorithm.

.TP
.B \-minint
.I
value

(min-update-interval)
.br
Minimum interval between link updates, i.e. hold-down timer.

.TP
.B \-mintime
.I
value

(min-sim-time)
.br
Minimum simulation time.

.TP
.B \-monlink
.br
(monitor-link)
.br
Monitor and log link state over time.  List of links to monitor is
specified in
.I
rsmonitor.dat

.TP
.B \-multipolicy [MULTI | FIRST | UNIQ]
(multi-route-policy)
.br
Multi-path routing policy when using SPT routing-algorithm option.
MULTI extracts a path with the option of checking link-feasibility and
backtracking.  UNIQ extracts a path according to the uniq-route-policy
option.  FIRST does no multi-path route extraction.

.TP
.B \-netmodel [DUAL | SINGLE] 
(network-model)
.br
DUAL is the hybrid routing scheme which applies dynamic routing to a
subset of flows using the WIDE_SHORT routing algorithm and
shortest-path to the remainder using SPT.  SINGLE applies the
algorithm specified by the routing-algorithm option to all flows.

.TP
.B \-netsplit
.I
value

(network-split) .br Network capacity split for short/long traffic.
The
.I value
sets the proportion of capacity placed on the short
partition.

.TP
.B \-pdrecomp
(periodic-recomputations)
.br
Use periodic route recomputation (in precomputed routing)

.TP
.B \-printtop
.br
(print-topology-stats)
.br
Print topology stats and exit

.TP
.B \-prune [DEFAULT_PRUNE | NO_PRUNE]
(prune-policy)
.br
Prune policy when using SPT routing-algorithm option.  Pruning removes
seemingly infeasible links from the route computation.

.TP
.B \-prunerecomp
.br
(prune-on-recomputation)
.br
Prune links that failed in signaling when recomputing routes (in
precomputed routing)

.TP
.B \-ralg [SPT | WIDE_SHORT]
(routing-algorithm)
.br
Sets the routing algorithm for the SINGLE network-model.  SPT is a
Dijkstra-based algorithm and WIDE_SHORT is the Bellman-Ford algorithm
by Guerin.

.TP
.B \-rcskew
.I
value

(recomputation-skew)
.br
Route recomputation period skew (in precomputed routing).

.TP
.B \-recomp
.I
value

(recomputation-interval)
.br
Route recomputation period (in precomputed routing).

.TP
.B \-reextsig
.br
(reextract-sig-failure)
.br
Reextract a new route and try again on set-up failure.

.TP
.B \-refresh
.I
value

(refresh-interval)
.br
Link-state update refresh period.

.TP
.B \-requests
.I
value

(min-requests)
.br
Minimum number of flow requests in simulation.

.TP
.B \-rgalpha
.I
value

(rand-graph-alpha)
.br
Random graph model parameter in Waxman's RG2 model.  Controls the
number of edges.

.TP
.B \-rgasym
(rand-graph-asymmetric)
.br
Use asymmetric random graphs (usually not used).

.TP
.B \-rgbeta
.I
value

(rand-graph-beta)
.br
Random graph model parameter in Waxman's RG2 model.  Controls the
ratio of long to short edges.


.TP
.B \-rgmethod [RG2 | PURE]
(rand-graph-method)
.br
Random graph generation method: RG2 is Waxman's model.  PURE is a
simple coin flip to decide an edge using rand-graph-alpha value as the
probability of having an edge.

.TP
.B \-rgplane
.I
value

(rand-graph-planesize)
.br
Random graph parameter for Euclidean plane dimension -- used in
Waxman's RG2 model.

.TP
.B \-route [ON_DEMAND | PRECOMP]
(routing-policy)
.br
Routing policy: on-demand (ON_DEMAND) or precomputed (PRECOMP)

.TP
.B \-rskew
.I
value

(refresh-skew)
.br
Link update refresh period skew.

.TP
.B \-seedval
.I
value

(random-seed)
.br
Random number generator seed (should be a long int).

.TP
.B \-simtol
.I
value

(sim-tolerance)
.br
Simulation confidence tolerance.

.TP
.B \-spread
(bandwidth-spread)
.I
value

.br
Width of bandwidth distribution (proportional to the mean value).

.TP
.B \-timeofday [MORNING | AFTERNOON | EVENING]
(time-of-day)
.br
Time of day: MORNING, AFTERNOON, EVENING.  Used with the ARR_FILE
option.

.TP
.B \-tmultiplier
.I
value

(traffic-multiplier)
.br
Scaling factor for traffic matrix arrival distribution scale
parameters.

.TP
.B \-topology [kncube | file | full
.B | rand]

(topology-type)
.br
Type of topology to use: k-ary n-cube (kncube), user file specified
(file), fully-connected (full), or random graph (rand).  Note
lowercase.

.TP
.B \-topsize
.I
value

(topology-size)
.br
Number of nodes in fully-connected topology.

.TP
.B \-trigger
.I
value

(update-trigger)
.br
Link-state update trigger threshhold.

.TP
.B \-trigroute
.br
(trigger-recomp-routefail)
.br
Trigger recomputation on routing failure.

.TP
.B \-trigsig
.br
(trigger-recomp-sigfail)
.br
Trigger recomputation on set-up failure.

.TP
.B \-uniqpolicy [SRCHASH | RANDOM]
(uniq-route-policy)
.br
Policy to use with UNIQ multi-route-policy option. SRCHASH picks a
route based on source node id.  RANDOM picks randomly among multiple
routes.

.TP
.B \-version
.br
(print-version)
.br
Print the routesim version and exit

.TP
.B \-warmup
.I
value

(min-warmup)
.br
Minimum warmup-phase flow requests

.TP
.B \-warmupprop
.I
value

(warmup-proportion)
.br
Minimum proportion of total requests that must be warmup-phase.

.TP
.B \-warmuptol
.I
value

(warmup-tolerance)
.br
Confidence interval tolerance for ending warmup phase.

.TP
.B \-wsmaxhops
.I
value

(wide-short-maxhops)
.br
Maximum allowed hopcount (for WIDE_SHORT alg)

.\" .TP
.\" .B \-conf
.\" .I
.\" configuration file
.\" (configuration-file)
.\" .br
.\" Name of configuration file

.SH "EXIT STATUS"
.I routesim
returns 0 upon successful completion, and 1 otherwise.

.SH FILES
.PD 0
.TP 20
.B rstopology.dat
.LP
.B rstopology.dat.out
.LP
.B rstraffic.dat
.LP
.B holdcdf.dat
.LP
.B rsmonitor.dat
.LP
.B rslinkmonitor.log
.LP
.B routesim.log
.LP
.B configuration file

.SH "SEE ALSO"

.BR rsfiles(l)

.SH DIAGNOSTICS

.I routesim
has extensive support for tracing, with different trace levels that
offer progressively more debugging  and monitoring information.
Changing the trace level, however, requires recompilation of the
simulator.  The default trace level, RS_TRACE_ERRORS, prints
information on standard error only for error conditions.


.SH BUGS
.PD
.I routesim
has a ridiculous number of parameters.
.PP
.I routesim
does not do enough sanity checking of options at startup.
.PP
There are probably some other bugs.

.SH AUTHOR

Anees Shaikh (University of Michigan, AT&T Labs--Research, 1997-99)

