/* $Id: rsdefs.h,v 3.18 1999/01/14 18:13:25 ashaikh Exp $ */

#ifndef _rsdefs_h_
#define _rsdefs_h_

#include "topology.h"
#include <limits.h>

/********************************************************************************/
/* Call-related types and definitions */

/* call request return codes */
#define REQUEST_FAILURE	    16
#define REQUEST_SUCCESS	    17

/* signalling return codes */
#define	    SIGNALLING_SUCCESS	18
#define	    SIGNALLING_FAILURE	19
#define	    SIG_SUCCESS		20
#define	    SIG_NO_BANDWIDTH	21
#define	    SIG_LINK_DOWN	22
#define	    SIG_MAX_REACHED	23

#define	    MAX_CRANKBACK_ATTEMPTS	2
#define	    MAX_SIG_ATTEMPTS_PER_CALL	(MAX_CRANKBACK_ATTEMPTS + 1)
#define	    DEFAULT_REROUTE_ATTEMPTS	50

typedef struct {
  int	    callid;	    /* call identifier		*/
  topology_t *network;	    /* network that the call is currently on */
  double    gentime;	    /* generation time		*/
  double    length;	    /* duration			*/
  double    bandw;	    /* bandwidth requirement    */
  int	    source;	    /* source			*/
  int	    dest;	    /* destination		*/
  int	    minhops;	    /* # nodes in minhop route  */
  int	    routelen;	    /* # nodes in route (w/src) */
  // New field to store multiple routes.
  // New field to store the index to extract the route later.
  // Needed for HSF.
  // Too much memory ??
  // Ganga 03/08/2000

  int       **allroutes;    /* store multiple routes for HSF */
  int       routeindex;     /* to store the index of the path extracted */
  int	    *route;	    /* nodes in the route	*/
  link_t    **links;	    /* links in the route	*/
  int	    routecomps;	    /* number of route computations */
  int	    extractions;    /* number of extraction attempts */  
  int	    sigattempts;    /* number of signalling attempts */
  int	    reroute;	    /* flag to indicate whether call has been rerouted */
  link_t    **failedlinks;  /* links that failed sig.	*/
} call_t;

typedef struct _termination_t {
  double    termtime;
  call_t    *call;
  topology_t	*network;
} termination_t;

typedef struct _update_t {
  double    updatetime;
  double    period;
  topology_t *network;
} update_t;

typedef struct _recomp_t {
  double    recomptime;
  int	    source;
  double    period;
  topology_t *network;  
} recomp_t;

typedef enum {DUAL, SINGLE} netmodel_t;

/********************************************************************************/
/* Traffic definitions */

typedef enum {MORNING, AFTERNOON, EVENING} timeofday_t;
typedef enum {ARR_UNIFORM, ARR_FILE} arrivalspec_t;
typedef enum {WEIBULL, POISSON} arrdist_t;
typedef enum {EXP, PARETO, USERDEF} holdspec_t;


/********************************************************************************/
/* Miscellaneous definitions */

#define RSLOGFILE	"routesim.log"
#define LINKMONLOGFILE	"rslinkmonitor.log"


#endif
