/* $Id: topologydefs.h,v 1.2 1999/06/02 21:14:46 ashaikh Exp $ */

#ifndef _topologydefs_h_
#define _topologydefs_h_

#include "stat.h"
#include "rsstatlist.h"
#include "rsroutedefs.h"


/********************************************************************************/

/* Represent graph as an adjacency matrix.  Each entry is a pointer
 * to a link structure that defines the link between the two nodes.
 * Link structures contain metric structures that hold current and
 * "old" values to represent stale link metric information.
 */

/* link parameters */
typedef struct _linkmetric_t {
  double  current;
  double  last;
} linkmetric_t;

/*  link structure */

typedef struct _link_t {
  int		id;		/* link identifier */
  int		from;		/* node id of "from" node */
  int		to;		/* node it of "to" node */
  int		pendingupdate;	/* flag to indicate whether an update is pending */
  double	lastupdtime;	/* last time link metrics were updated */
  double	capacity;	/* max available bandwidth */
  double	adminweight;	/* administrative weight */
  double	propdelay;	/* fixed propagation delay */
  linkmetric_t	availbw;	/* current available bandwidth */
  linkmetric_t	delay;		/* current delay */
  linkmetric_t	hopcount;	/* hop count for this link (nominally 1) */
  linkmetric_t	lossratio;	/* current loss ratio */
  linkmetric_t	cost;		/* current link cost */
  int		numcalls;	/* current number of calls */
  int		callsceil;	/* number of calls accepted on this link */
  int		maxcalls;	/* max calls this link can handle */
  void		**calls;	/* pointers to calls using this link */
  int		active;		/* boolean to indicate whether to use this link */
  int		down;		/* boolean to indicat whether this link gone down */
  int		monitor;	/* boolean to indicate whether we should monitor this link */
  simpleratio_t	util;		/* link utilization statistic */  
  double	lastutilupd;	/* last time the utilization on this link was recorded */
} link_t;

typedef enum {TOP_KNCUBE, TOP_FILE, TOP_FULL, TOP_RAND} topologytype_t;

/* this may not be the best place for this */
typedef enum {NON_BLOCKING, BLOCKING} blockingpolicy_t;

typedef struct {
  topologytype_t    type;
  link_t	    ***links;
  int		    **hoptable;
  int		    numnodes;
  int		    numlinks;
  int		    diameter;
  simpleratio_t	    hopcount;
  simpleratio_t	    degree;
  simpleratio_t	    linkcapacity;
  blockingpolicy_t  blkpolicy;
  rsstat_t	    *stats;		/* statistics package for the network */
  routealg_t	    routingalg;
  void		    **routetable;	/* source-specific routing tables */
  void		    *routepolicies;
  void		    *info;
} topology_t;

typedef enum {RG2, PURE} randgraph_t;



#endif
