#!/bin/perl
#$Id: rsgraph.pl,v 1.2 1998/02/11 19:58:51 ashaikh Exp ashaikh $
#
# A. Shaikh
#
#   Construct xmgr graphs automatically with routesim data
#
#
#

#use lib "/home/shaikh/routesim/";
require "/opt/users/ashaikh/Research/routesim/scripts/Xmgr_parms.pm";
use File::Basename;

%labels = ();
$labels{calls} = "Total calls created";
$labels{failures} = "Failed calls";
$labels{routefail} = "Routing failures";
$labels{sigfail} = "Signalling failures";
$labels{routefailrate} = "Routing failure rate";
$labels{sigfailrate} = "Signalling failure rate";
$labels{callblk} = "Flow blocking probability";
$matchstring{callblkdev} = "Flow blocking rate std. dev.";
$labels{bwblk} = "Bandwidth blocking rate";
$labels{hcblk} = "Hopcount blocking rate";
$labels{hcbwblk} = "Hopcount-bandwidth blocking rate";
$labels{hcbwcov} = "Hopcount-bandwidth blocking covariance";
$labels{util} = "Measured utilization";
$labels{minhoprat} = "Hopcount to minhop ratio";
$labels{avghopdiff} = "Avg. hopcount difference";
$labels{maxhopdiff} = "Max hopcount difference";
$labels{linkupdrate} = "Global link update rate";
$labels{plinkupd} = "Refresh link updates";
$labels{tlinkupd} = "Triggered link updates";
$labels{dlinkupd} = "Deferred link updates";
$labels{elinkupd} = "Extraneous updates";
$labels{injload} = "Total injected load";
$labels{residual} = "Avg. residual bottleneck bandwidth";
$labels{updinterval} = "Avg. link state update interarrival";
$labels{hop2} = "2-hop Blocking rate";
$labels{hop3} = "3-hop Blocking rate";
$labels{hop4} = "4-hop Blocking rate";
$labels{hop5} = "5-hop Blocking rate";
$labels{hop6} = "6-hop Blocking rate";
$labels{sigblk} = "Blocking due to signalling failures";
$labels{routeblk} = "Blocking due to routing failures";
$labels{sigcallblk} = "Signalling blocking probability";
$labels{routecallblk} = "Routing blocking probability";
$labels{routecomprate} = "Route computation rate per node";
$labels{sigratepernode} = "Per-node signalling rate";
$labels{sigrateperreq} = "Per-request signalling rate";
$labels{sigratepersigcall} = "Signalling rate per signalled request";
$labels{sigattpersucccall} = "Signalling attempts per admitted request";
$labels{sigattperfailcall} = "Signalling attempts per blocked request";
$labels{extattpercall} = "Extraction attempts per request";
$labels{extattpersucccall} = "Extraction attempts per admitted request";
$labels{extattperfailcall} = "Extraction attempts per blocked request";
$labels{linkutil} = "Avg. link utilization";
$labels{linkunderutil} = "Avg. time proportion underutilized";
$labels{linkoverutil} = "Avg. time proportion overutilized";
$labels{linkviolutil} = "Avg. time proportion in violation";
$labels{linkflowunderutil} = "Time avg. number of flows underutilized";
$labels{linkflowoverutil} = "Time avg. number of flows overutilized";
$labels{linkflowviolutil} = "Time avg. number of flows in violation";
$labels{avgunderutil} = "Avg. underutilization";
$labels{avgoverutil} = "Avg. overutilization";
$labels{avgviolutil} = "Avg. violation";
    

##########################################################################################

$numsets = 0;
&readCmdLine;
print "num. sets: ". $numsets . "\n";
$outfile = "/tmp/rs.par";

&getMetric;
&printSets;
&openOutfile;
&writeGlobalParms;
&writeGraphParms;
&writeSetParms;
&writeAxesParms;
&writeLegendParms;
close OUTPUT;
&runXmgr;


##########################################################################################
sub readCmdLine
{
    local ($i, $arg, );


    # some checks on the command line
    if ($#ARGV < 0) {
	print "No data files specified!\n";
	exit;
    }    
    if (($#ARGV + 1) % 2 != 0) {
	print "Usage: rsgraph.pl <file1> \"<string1>\" .... <fileN> \"<stringN>\"\n";
	print "\tfileX -- filename for set X\n";
	print "\tstringX -- graph legend string for set X\n";
	exit;
    }

    for ($i = 0; $i <= $#ARGV; $i += 2) {
	$sets[$numsets]->{filename} = $ARGV[$i];
	$sets[$numsets]->{legend} = $ARGV[$i + 1];
	$numsets++;
    }
}

sub printSets
{
    local ($i);

    foreach $i (0 .. $#sets) {
	print "set $i:\n";
	print "\tfile: $sets[$i]->{filename}\n";
	print "\tlegend: $sets[$i]->{legend}\n";
    }
}
  
sub getMetric
{
    local ($i, $first, $curr, $prev);

    $sets[0]->{filename} =~ /([a-z]+)\.dat/;    
    $prev = $1;
    foreach $i (1 .. $#sets) {
	$sets[$i]->{filename} =~ /([a-z]+)\.dat/;
	$curr = $1;
#	if ($curr ne $prev) {
#	    die "Metrics don't match! ($curr != $prev)\n";
#	}
#	else {
#	    $prev = $curr;
#	}
    }
    $metric = $prev;
    print "metric: $labels{$metric} ($metric)\n";
}

sub openOutfile
{
    open (OUTPUT, ">$outfile") || die "Could not open $outfile for writing\n";
}

##########################################################################################
sub writeGlobalParms
{
    print OUTPUT "\# xmgr parameter file\n";
    print OUTPUT "\#\n";
    print OUTPUT "\#\n";
    print OUTPUT "page layout free\n";
    print OUTPUT "ps linewidth begin 1\n";
    print OUTPUT "ps linewidth increment 2\n";
    print OUTPUT "hardcopy device 2\n";
    print OUTPUT "page 5\n";
    print OUTPUT "page inout 5\n";
    print OUTPUT "link page off\n";
    print OUTPUT "default linestyle 1\n";
    print OUTPUT "default linewidth 1\n";
    print OUTPUT "default color 1\n";
    print OUTPUT "default char size 0.8000000\n";
    print OUTPUT "default font 4\n";
    print OUTPUT "default font source 0\n";
}

sub writeGraphParms
{
    local ($parm, $subparm, $prefix);
    
    print OUTPUT "with g0\n";
    print OUTPUT "g0 on\n";
    print OUTPUT "g0 label off\n";
    print OUTPUT "g0 hidden false\n";
    print OUTPUT "g0 type xy\n";
    print OUTPUT "g0 autoscale type AUTO\n";
    print OUTPUT "g0 fixedpoint off\n";
    $prefix = "  ";
    foreach $parm (keys %graphparms) {
	if (keys %{$graphparms{$parm}} == ()) {
	    print OUTPUT $prefix . "$parm \"\"\n";
	}
	else {
	    foreach $subparm (keys %{$graphparms{$parm}}) {
		print OUTPUT $prefix . "$parm $subparm $graphparms{$parm}->{$subparm}\n";
	    }
	}
    }
}    

sub writeSetParms
{
    local ($i, $prefix, $parm, $subparm);

    $prefix = "  sets ";
    foreach $parm (keys %setsparms) {
	print OUTPUT $prefix . "$subparm $setparms{$parm}->{$subparm}\n";
    }
    
    foreach $i (0 .. $#sets) {
	$prefix = "  s$i ";
#use only symbols 1--11
	print OUTPUT $prefix . "symbol " .  (($i % 10) + 2) . "\n";
	print OUTPUT $prefix . "xyz 0.000000, 0.000000\n";
	print OUTPUT $prefix . "comment \"$sets[$i]->{filename}\"\n";
    }
}

sub writeAxesParms
{
    #ticks, ticklabels

    local ($i, $prefix, @parms, $parm, $subparm);



    $prefix = "  xaxis ";
#    print OUTPUT $prefix . "tick on\n";
    print OUTPUT $prefix . "tick in\n";
    @parms = ("tick", "ticklabel", "label", "bar");
    foreach $parm (@parms) {
	foreach $subparm (sort keys %{$axes{$parm}}) {
	    print OUTPUT $prefix . "$parm $subparm " . $axes{$parm}->{$subparm}. "\n";
	}
    }
    print OUTPUT "  xaxis label \"\"\n";
    $prefix = "  yaxis ";
#    print OUTPUT $prefix . "tick on\n";
    print OUTPUT $prefix . "tick in\n";
    @parms = ("tick", "ticklabel", "label", "bar");
    foreach $parm (@parms) {
	foreach $subparm (sort keys %{$axes{$parm}}) {
	    print OUTPUT $prefix . "$parm $subparm " . $axes{$parm}->{$subparm}. "\n";
	}
    }    
    print OUTPUT "  yaxis label \"$labels{$metric}\"\n";
}

sub writeLegendParms
{
    # legends and frame parameters

    local ($i, $prefix);

    $prefix = "  legend ";
    print OUTPUT $prefix . "on\n";

    foreach $i (keys %legends) {
	print OUTPUT $prefix . "$i $legends{$i}\n";
    }
    foreach $i (0 .. $#sets) {
	print OUTPUT $prefix . "string $i \"" . $sets[$i]->{legend} . "\"\n";
    }

    $prefix = "  frame ";
    print OUTPUT $prefix . "on\n";
    foreach $i (keys %frame) {
	print OUTPUT $prefix . "$i $frame{$i}\n";
    }
}

##########################################################################################

sub runXmgr
{
    local ($i, $cmdline);
    
    $cmdline = "xmgr -autoscale xy -p $outfile ";
    foreach $i (0 .. $#sets) {
	$cmdline = $cmdline . $sets[$i]->{filename} . " ";
    }
    print $cmdline. "\n";
    system ("$cmdline &");
}
