#!/bin/perl
# $Id$
# $Source$
#
#
# This script extracts data from a logfile generated by monitoring a single
# link over time.
#
#

# some default filename info

$infile = "rslinkmonitor.log";


# arrays and counters
%events = ();
%time = ();
%link = ();
%callid = ();
%bandw = ();
%occupancy = ();
%avail = ();
%last = ();
%inaccuracy = ();
%adv = ();
%util = ();
%interval = ();
$arrindex = 0;
$terminidex = 0;
$failindex = 0;
$updindex = 0;
$refindex = 0;
$pendindex = 0;
$utilindex = 0;


##########################################################################################

&makeDirs;
&processFiles;

##########################################################################################
sub makeDirs {

    $outdir = "datafiles";
    $subdir = "timeseries";

    # check for datafiles subdirectory
    if (! -d $outdir || ! -e $outdir) {
	die "Please create ./$outdir directory!\n";
    }
    # make the directory for timeseries data
    mkdir ("$outdir/$subdir", 0750);
}

##########################################################################################
sub processFiles {

    my ($al, $pd, $inprefix, $insuffix, $outprefix, $outsuffix, $file);

    $file = "rslinkmonitor.log";
    &readFile ($file);
    $file = "monitordat.out";
    &processEvents ($file);
    &processMetrics ($file);
}

sub readFile {

    my ($infile, $prefix, $suffix);

    $infile = $_[0];

    %events = ();
    %time = ();
    %link = ();
    %callid = ();
    %bandw = ();
    %occupancy = ();
    %avail = ();
    %last = ();
    %inaccuracy = ();
    %adv = ();
    %util = ();
    %interval = ();

    $arrindex = 0;
    $termindex = 0;
    $failindex = 0;
    $updindex = 0;
    $refindex = 0;
    $pendindex = 0;
    $utilindex = 0;

    open (RESULTS, "<$infile") || die "Could not open $infile for reading\n";
    while (<RESULTS>) {
	chop;
	if (/^\s*([a-z])\s([0-9]*\.*[0-9]*)\s([0-9]+\->[0-9]+)/) {
	    @fields = split(/\s+/);

	    if ($fields[0] eq 'a') {
		$index = "arr" . $arrindex;
		$events{arrivals}->[$arrindex] = $index;
		$time{$index} = $fields[1];
		$link{$index} = $fields[2];
		$callid{$index} = $fields[3];
		$bandw{$index} = $fields[4];
		$occupancy{$index} = $fields[5];
		$avail{$index} = $fields[6];
		$last{$index} = $fields[7];
		$inaccuracy{$index} = $fields[8];
		$arrindex++;
	    }
	    if ($fields[0] eq 't') {
		$index = "term" . $termindex;
		$events{terminations}->[$termindex] = $index;
		$time{$index} = $fields[1];
		$link{$index} = $fields[2];
		$callid{$index} = $fields[3];
		$bandw{$index} = $fields[4];
		$occupancy{$index} = $fields[5];
		$avail{$index} = $fields[6];
		$last{$index} = $fields[7];
		$inaccuracy{$index} = $fields[8];
		$termindex++;
	    }
	    if ($fields[0] eq 'f') {
		$index = "fail" . $failindex;
		$events{sigfailures}->[$failindex] = $index;
		$time{$index} = $fields[1];
		$link{$index} = $fields[2];
		$failindex++;
	    }
	    if ($fields[0] eq 'u') {
		$index = "trig" . $updindex;
		$events{trigupdates}->[$updindex] = $index;
		$time{$index} = $fields[1];
		$link{$index} = $fields[2];
		$adv{$index} = $fields[3];
		$interval{$index} = $fields[4];
		$updindex++;
	    }
	    if ($fields[0] eq 'r') {
		$index = "ref" . $refindex;
		$events{refreshes}->[$refindex] = $index;
		$time{$index} = $fields[1];
		$link{$index} = $fields[2];
		$adv{$index} = $fields[3];
		$interval{$index} = $fields[4];
		$refindex++;
	    }
	    if ($fields[0] eq 'p') {
		$index = "pend" . $pendindex;
		$events{pendings}->[$pendindex] = $index;
		$time{$index} = $fields[1];
		$link{$index} = $fields[2];
		$adv{$index} = $fields[3];
		$interval{$index} = $fields[4];		
		$pendindex++;
	    }
	    if ($fields[0] eq 'b') {
		$index = "util" . $utilindex;
		$events{utils}->[$utilindex] = $index;
		$time{$index} = $fields[1];
		$link{$index} = $fields[2];
		$util{$index} = $fields[3];
		$utilindex++;
	    }	    
	}
    }
}

##########################################################################################
sub processEvents {
    # record (time,event) tuples

    my ($i, $outdir, $outsubdir, $outfile, $e);

    $outdir = "datafiles/timeseries";
    # check for datafiles subdirectory
    if (! -d $outdir || ! -e $outdir) {
	die "Please create ./$outdir directory!\n";
    }
    # make the directory for timeseries data

    # create the data files for each event type
    foreach $e (keys %events) {
	$outsubdir = "./$outdir/$e";
	if (! -e $outsubdir) {
	    mkdir ($outsubdir, 0750);
	}
	$outfile = "$outsubdir/$_[0]-$e";
	if (-e $outfile) {
	    die "$outfile already exists ... exiting\n";
	}
	open (OUTPUT, ">$outfile") || die "Could not open $outfile for writing\n";

	foreach $i ( 0 .. $#{$events{$e}} ) {
	    $ev = $events{$e}->[$i];

	    print OUTPUT "$time{$ev} 0\n";
	}
	close OUTPUT;
    }
}

##########################################################################################

sub processMetrics {

    my ($ev, $outdir, $subdir, $outfile1, $outfile2, $outfile3, $outfile4, @evs);
    
    # now create some more interesting data
    $outdir = "datafiles/timeseries";
    $subdir = "timeseries";

    # check for datafiles subdirectory
    if (! -d $outdir || ! -e $outdir) {
	die "Please create ./$outdir directory!\n";
    }
    
    ### occupancy, available, and inaccuracy
    if (! -e "$outdir/occupancy") {
	mkdir ("$outdir/occupancy", 0750);
    }
    $outfile1 = "$outdir/occupancy/$_[0]-occupancy";
    open (OUTPUT1, ">$outfile1") || die "Could not open $outfile1 for writing\n";

    if (! -e "$outdir/avail") {
	mkdir ("$outdir/avail", 0750);
    }
    $outfile2 = "$outdir/avail/$_[0]-avail";
    open (OUTPUT2, ">$outfile2") || die "Could not open $outfile2 for writing\n";    

    if (! -e "$outdir/inaccuracy") {
	mkdir ("$outdir/inaccuracy", 0750);
    }
    $outfile3 = "$outdir/inaccuracy/$_[0]-inaccuracy";
    open (OUTPUT3, ">$outfile3") || die "Could not open $outfile3 for writing\n";

    if (! -e "$outdir/last") {
	mkdir ("$outdir/last", 0750);
    }
    $outfile4 = "$outdir/last/$_[0]-last";
    open (OUTPUT4, ">$outfile4") || die "Could not open $outfile4 for writing\n";    
    @evs = ();
    push(@evs, @{$events{arrivals}});
    push (@evs, @{$events{terminations}});
    foreach $ev (sort by_time @evs) {
	print OUTPUT1 "$time{$ev} $occupancy{$ev}\n";
	print OUTPUT2 "$time{$ev} $avail{$ev}\n";
	print OUTPUT3 "$time{$ev} $inaccuracy{$ev}\n";
	print OUTPUT4 "$time{$ev} $last{$ev}\n";
    }
    close OUTPUT1;
    close OUTPUT2;
    close OUTPUT3;
    close OUTPUT4;
    ### advertised
    if (! -e "$outdir/adv") {
	mkdir ("$outdir/adv", 0750);
    }
    $outfile1 = "$outdir/adv/$_[0]-adv";
    open (OUTPUT1, ">$outfile1") || die "Could not open $outfile for writing\n";
    @evs = ();
    push (@evs, @{$events{trigupdates}});
    push (@evs, @{$events{refreshes}});
    push (@evs, @{$events{pendings}});
    foreach $ev (sort by_time @evs) {
	print OUTPUT1 "$time{$ev} $adv{$ev}\n";
    }
}
    
    
sub by_time {

    $time{$a} <=> $time{$b};

}
