/* $Id: bfalg.c,v 2.4 1999/01/25 21:32:14 ashaikh Exp $ */

static const char rcsid[] = "$Id: bfalg.c,v 2.4 1999/01/25 21:32:14 ashaikh Exp $";

/*  Description:
 *
 *	Computes a shortest-widest path routing table based on the algorithm by Guerin, et. al.
 *
 */

#include "bfalg.h"
#include "topology.h"
#include "rsdefs.h"
#include "rsroute.h"
#include "routesim.h"
#include "rstrace.h"
#include <stdlib.h>
#include <string.h>


typedef struct {
  int	    *list;
  int	    num;
} slist_t;

/********************************************************************************/
int
bfInit (bftab_t **newtab, int source, int maxhops, int extrahops, topology_t *net)
{
  int	    i, numnodes;
  bftab_t   *bftab;
  int	    tabwidth;

  if (net == (topology_t *) 0) {
    RS_PRINT_ERROR ("bfInit: Invalid network\n");
    return (-1);
  }

  bftab = (bftab_t *) malloc (sizeof(bftab_t));
  if (bftab == (bftab_t *) 0) {
    RS_PRINT_SOFT_ERROR ("bfInit: Table allocation failed\n");
    return (-1);
  }

  numnodes = getNumNodes (net);
  /* allocate the table */
  bftab->tt = (bfentry_t **) malloc (sizeof(bfentry_t *) * numnodes);	/* N rows */
  if (bftab->tt == (bfentry_t **) 0) {
    RS_PRINT_SOFT_ERROR ("bfInit: Table rows allocation failed\n");
    return (-1);
  }
    
  /* use N - 1 if maxhops is not defined */  
  maxhops <= 0 ? (tabwidth = (numnodes)) : (tabwidth = (maxhops + 1));
  /* table width include #hops plus 1 for zero-hops entry */
  for (i = 0; i < numnodes; i++) {
    bftab->tt[i] = (bfentry_t *) malloc (sizeof(bfentry_t) * tabwidth);
    if (bftab->tt[i] == (bfentry_t *) 0) {
      RS_PRINT_SOFT_ERROR ("bfInit: Table column allocation failed\n");
      return (-1);
    }
  }

  bftab->network = net;
  bftab->hoplimit = tabwidth - 1;
  bftab->extrahops = extrahops;
  bftab->source = source;
  bftab->valid = 0;
  bftab->comptime = -1.0;
  bftab->computations = 0;
  
  *newtab = bftab;
      
  return (0);
}
/********************************************************************************/
int
bfReset (bftab_t *bftab)
{
  int	i, j, numnodes;

  if (bftab == (bftab_t *) 0) {
    RS_PRINT_ERROR ("bfReset: Invalid BF table pointer\n");
    return (-1);
  }

  numnodes = getNumNodes (bftab->network);
  
  for (i = 0; i < numnodes; i++) {
    for (j = 0; j < (bftab->hoplimit + 1); j++) {
      bftab->tt[i][j].parent = -1;
      bftab->tt[i][j].bw = (double) -INF;
    }
  }
  bftab->valid = 0;

  return (0);
}
      
/********************************************************************************/
int
bfExec (bftab_t *bftab, double currenttime)
{

  int		source, numnodes;
  int		i, j, h;
  slist_t	*s1, *s2, *sprev, *snew, *tmp;
  double	minbw;


  if (bftab == (bftab_t *) 0) {
    RS_PRINT_ERROR ("bfExec: Invalid BF table pointer\n");
    return (-1);
  }

  if (bftab->source < 0) {    /* bftab not initialized yet */
    RS_PRINT_ERROR ("bfExec:  BF table not initialized properly\n");
    return (-1);
  }
  source = bftab->source;
  numnodes = getNumNodes(bftab->network);
  
  /* Initialize data structures */
  s1 = (slist_t *) malloc (sizeof(slist_t));
  s2 = (slist_t *) malloc (sizeof(slist_t));
  if ( (s1 == (slist_t *) 0) || (s2 == (slist_t *) 0) ) {
    RS_PRINT_SOFT_ERROR ("bfExec:  Error allocating slists\n");
    return (-1);
  }
  s1->list = (int *) malloc (sizeof(int) * numnodes);
  s2->list = (int *) malloc (sizeof(int) * numnodes);
  if ( (s1->list == (int *) 0) || (s2->list == (int *) 0) ) {
    RS_PRINT_SOFT_ERROR ("bfExec:  Error allocating slist arrays\n");
    return (-1);
  }
  s1->num = 0;
  s2->num = 0;
  memset (s1->list, 0, sizeof(int)*numnodes);
  memset (s2->list, 0, sizeof(int)*numnodes);
  sprev = s1;	/* s1 is sprev */
  snew = s2;

  /* some initialization for 0-hop and 1-hop entries */
  for (i = 0; i < numnodes; i++) {
    bftab->tt[i][0].bw = (double) -INF;
    bftab->tt[i][0].parent = -1;
    bftab->tt[i][1].bw = (double) -INF;
    bftab->tt[i][1].parent = -1;
  }
  bftab->tt[source][0].bw = (double)INF;    /* infinite bw from source to itself */

  /* for neighbors of source, i.e. 1-hop routes */
  for (j = 0; j < numnodes; j++) {
    if ( (source == j) || (bftab->network->links[source][j] == (link_t *) 0) )
      continue;
    if (processLinkUpdates (currenttime, bftab->network->links[source][j], bftab->network, "(routing)")) {
      RS_PRINT_ERROR ("bfExec: processLinkUpdates failed\n");
      return (-1);
    }    
    if (bftab->network->links[source][j]->availbw.current > bftab->tt[j][1].bw) {
      bftab->tt[j][1].bw = bftab->network->links[source][j]->availbw.current;
      bftab->tt[j][1].parent = source;
      sprev->list[j] = 1;
      sprev->num++;
    }
  }

  for (h = 2; h <= bftab->hoplimit; h++) {
    /* reset snew array */
    memset (snew->list, 0, sizeof(int)*numnodes);
    snew->num = 0;
    for (i = 0; i < numnodes; i++) {
      /* h+1-hop parent&bw is tentatively same as h-hop parent&bw */      
      bftab->tt[i][h].bw = bftab->tt[i][h-1].bw;
      bftab->tt[i][h].parent = bftab->tt[i][h-1].parent;
    }
    for (i = 0; i < numnodes; i++) {
      /* for all vertices i in S_prev */
      if (sprev->list[i]) {
	/* look at neighboring links of node i */
	for (j = 0; j < numnodes; j++) {
	  if ( (i == j) || (bftab->network->links[i][j] == (link_t *) 0) )
	    continue;
	  if (processLinkUpdates (currenttime, bftab->network->links[i][j], bftab->network, "(routing)")) {
	    RS_PRINT_ERROR ("bfExec: processLinkUpdates failed\n");
	    return (-1);
	  }
	  /* calculate minbw */
	  if (bftab->tt[i][h-1].bw < bftab->network->links[i][j]->availbw.last)
	    minbw = bftab->tt[i][h-1].bw;
	  else
	    minbw = bftab->network->links[i][j]->availbw.last;

	  if (minbw > bftab->tt[j][h].bw) { /* need to update */
	    bftab->tt[j][h].bw = minbw;
	    bftab->tt[j][h].parent = i;
	    snew->list[j] = 1;
	    snew->num++;
	  }
	}
      }
    }
    /* swap sprev and snew arrays */
    tmp = sprev;
    sprev = snew;
    snew = tmp;
    /* check if anything in sprev changed -- if not, advance h */
    if (sprev->num == 0) {
	h++;
    }
  }

  bftab->valid = 1;
  bftab->comptime = currenttime;
  bftab->computations++;
  free (s1->list);
  free (s2->list);
  free (s1);
  free (s2);

  return (0);
}

/********************************************************************************/
int
bfExtractPath (bftab_t *bftab, call_t *newcall, int **route, int *result)
{
  int	    *path;
  int	    len, current, hop;
  int	    source, dest;

  *route = (int *) 0;
  source = newcall->source;
  dest = newcall->dest;
  
  if ( (bftab == (bftab_t *) 0) || (bftab->source < 0) ) {
    RS_PRINT_ERROR ("bfExtractPath:  BF Table not initialized\n");
    *result = ROUTE_EXT_FAIL;
    return (-1);
  }

  len = bfGetPathLen (bftab, dest, newcall->bandw);

  if (len < 0) {
    RS_PRINT_MORE_EVENT ("bfExtractPath: bfGetPathLen did not find feasible path\n");
    *result = ROUTE_EXT_FAIL;
    return (0);
  }
  if (len == 0)	{ /* don't return a route if src and dest are the same */
    *result = ROUTE_EXT_SUCCESS;
    return (0);
  }

  path = (int *) malloc (sizeof(int)*(len + 1));  /* len + 1 for source */

  /* extract source */
  current = dest;
  for (hop = len; hop >= 0; hop--) {
    path[hop] = current;
    current = bftab->tt[current][hop].parent;
  }  

  *route = path;
  newcall->routelen = len + 1;	/* to include source node */
  *result = ROUTE_EXT_SUCCESS;
  return (0);


}
/********************************************************************************/
int
bfGetPathLen (bftab_t *bftab, int dest, double reqbw)
/* Return the length (number of hops) of the shortest feasible path.  If requested
 * bw is unspecified, returns length of shortest path with nonzero available bandwidth.
 */
{
  int		    i;
  int		    pos;
  int		    minhops, searchlimit;
  blockingpolicy_t  policy;

  if ( (bftab == (bftab_t *) 0) || (bftab->source < 0) ) {
    RS_PRINT_ERROR ("bfExtractPath:  BF Table not initialized\n");
    return (-1);
  }

  if (bftab->source == dest)	/* source to itself has infinite bw! */
    return (0);

  policy = bftab->network->blkpolicy;

  minhops = getMinHops (bftab->source, dest, bftab->network);
  /* make sure there is a minhop path */
  if (bftab->tt[dest][minhops].bw <= ((double)-INF)) {
    RS_PRINT_ERROR ("bfExtractPathLen:  No minhop path available!\n");
    return (-1);
  }

  /* compute how much of the table we can search */  
  if ( (bftab->extrahops < 0) || ((minhops + bftab->extrahops) > bftab->hoplimit) ) {
    searchlimit = bftab->hoplimit;
  }
  else {
    searchlimit = minhops + bftab->extrahops;
  }  

  pos = -1;
  for (i = 1; i <= searchlimit; i++) {   /* start at one hop */
    if (bftab->tt[dest][i].bw >= reqbw) {
      pos = i;
      break;
    }
  }
  
  if (pos == -1) { /* no feasible path found */
    if (policy == BLOCKING) {
#if RS_TRACE_LEVEL >= TRACE_MORE_EVENTS
      printf ("bfGetPathLen:  No feasible path found %d -> %d (bw: %.3f)\n",
	      bftab->source, dest, reqbw);
#endif
      return (-1);
    }
    else {  /* fall back to minhop path in NON-BLOCKING case */
      return (minhops);
    }
  }
  else {   /* found feasible path */
    return (pos);
  }
}

/********************************************************************************/
int
bfDestroy (bftab_t *bftab)
{
  int	i, numnodes;

  if ( bftab == (bftab_t *) 0) {
    RS_PRINT_ERROR ("bfDestroy:  BF Table not initialized\n");
    /* don't really need to return an error here */
    return (-1);
  }

  numnodes = getNumNodes (bftab->network);
  for (i = 0; i < numnodes; i++)
    free (bftab->tt[i]);

  free (bftab->tt);
  free (bftab);

  return (0);
}

/********************************************************************************/
void
bfPrintPath (bftab_t *bftab, int dest, double reqbw)
{
  int	len, current, hop;
  int	*path;

  if ( (bftab == (bftab_t *) 0) || (bftab->source < 0) ) {
    RS_PRINT_ERROR ("bfPrintPath:  BF Table not initialized\n");
    return;
  }

  len = bfGetPathLen (bftab, dest, reqbw);
  if (len < 0) {
    RS_PRINT_MORE_EVENT ("bfPrintPath: bfGetPathLen did not find valid path\n");
    return;
  }
  if (len == 0)	/* don't print anything if src and dest are the same */
    return;
  
  path = (int *) malloc (sizeof(int) * (len + 1));  /* len + 1 to keep a space for the source node */

  /* extract -- can't print as we traverse because this is the reverse path */
  current = dest;
  for (hop = len; hop >= 0; hop--) {
    path[hop] = current;
    current = bftab->tt[current][hop].parent;
  }

  /* print route */
  printf ("%d -> %d (%.3f): ", bftab->source, dest, bftab->tt[dest][len].bw);
  for (hop = 0; hop < len + 1; hop++) {
    printf ("%d ", path[hop]);
  }
  printf ("\n");

  free (path);

}
/********************************************************************************/	 
void
printBFTable (bftab_t *bftab)
{
  int	i, j;

  for (i = 0; i < bftab->network->numnodes; i++) {
    printf ("%2d: ", i);
    for (j = 0; j <= bftab->hoplimit; j++) {
      if (bftab->tt[i][j].bw == (double)INF) {
	printf ("(INF,%d) ", bftab->tt[i][j].parent);
      }
      else {
	if (bftab->tt[i][j].bw == (double)-INF) {
	  printf ("(-INF,%d) ", bftab->tt[i][j].parent);
	}
	else {
	  printf ("(%.3f,%d) ", bftab->tt[i][j].bw, bftab->tt[i][j].parent);
	}
      }
    }
    printf ("\n");
  }
}




