/* Description:
 *
 *    Used to compute the best path using the HSF algorithm.
 *
 */

#include "hsfalg.h"
#include "topology.h"
#include "rsdefs.h"
#include "rsroute.h"
#include "routesim.h"
#include "rstrace.h"
#include <stdlib.h>
#include <string.h>

/* hsfExec actually finds the existence of a path.
 * Ganga - 03/14/2000
 */

int hsfExec(call_t *newcall, double time, double bw, topology_t *network, int *result)
{
  link_t *currlink;
  
  int **allroutes;
  int kindex, nodeindex;
  int flag = 0;
  int nexthopindex, currindex;
  

  allroutes = newcall->allroutes;
  kindex = 0; nodeindex = 0;
  newcall->routeindex = -1;

  if(newcall->source == newcall->dest) {
    newcall->routeindex = 0;
    *result = ROUTE_CALC_SUCCESS;
    return (0);
  }

  /* Adding code for network cost
   * Ganga - 03/30/2000
   */

  for(kindex = 0; allroutes[kindex][0] != -1; kindex++) {
    for(nodeindex = 0; allroutes[kindex][nodeindex] !=-1; nodeindex++) {
      network->stats->nodesvisited++;
    }
  }


  
  for(kindex = 0; allroutes[kindex][0] != -1; kindex++) {
    nodeindex = 0;
    flag = 0;
    for(nodeindex = 0; allroutes[kindex][nodeindex] !=-1; nodeindex++) {
      currindex = allroutes[kindex][nodeindex];
      nexthopindex = allroutes[kindex][nodeindex+1];
      if(nexthopindex == -1) 
	break;      
      currlink = network->links[currindex][nexthopindex];
      
      if ((currlink->availbw.current) < bw) {
	flag = 1;
	break;
      }
    }
    if (!flag) {
      newcall->routeindex = kindex;
      *result = ROUTE_CALC_SUCCESS;
      return 0;
    }
  }  
  
  *result = ROUTE_CALC_FAIL;
  newcall->routeindex = -1;
  return 0;
}

/********************************************************************************/

int 
hsfExtractPath (call_t *newcall, int **route, int *result)
{
  int *path;
  int len, hop;
  int source, dest;
  int **allroutes;


  *route = (int *) 0;
  source = newcall->source;
  dest = newcall->dest;
  allroutes = newcall->allroutes;


  len = hsfGetPathLen (newcall);

  if (len < 0) {
    RS_PRINT_MORE_EVENT ("hsfEctractPath: hsfGetPathLen did not find feasible path\n");
    *result = ROUTE_EXT_FAIL;
    return (0);
  }
  
  if (len == 0) { /* don't return a route if src and dest are the same */
    *result = ROUTE_EXT_SUCCESS;
    return (0);
  }

  path = (int *) malloc (sizeof(int)*(len+1)); /* len +1 for source */

  /* extract path */

  for(hop = 0; allroutes[newcall->routeindex][hop] != -1; hop++) {
    path[hop] = allroutes[newcall->routeindex][hop];
      }

  *route = path;
  newcall->routelen = len + 1; /* to include source node */
  *result = ROUTE_EXT_SUCCESS;
  return (0);

} 

/********************************************************************************/

int 
hsfGetPathLen(call_t *newcall)
{
  int len;
  int hop;
  int routeindex;

  if ((newcall->source) == (newcall->dest)) {
    return (0);
  }

  routeindex = newcall->routeindex;
  hop = 0;
  while((newcall->allroutes[routeindex][hop]) != -1) {
    hop++;
  }
  len = hop - 1; /* dest and end marker and starts from 0*/
  return (len);
  
}





