/*$Id: rsopt.c,v 2.28 1999/06/02 20:34:00 ashaikh Exp $*/

static const char rcsid[] = "$Id: rsopt.c,v 2.28 1999/06/02 20:34:00 ashaikh Exp $";

#include <stdio.h>
#include <string.h>
#include "general.h"
#include "opt-parse.h"
#include "rsdefs.h"
#include "topology.h"
#include "rsroute.h"

static int ALWAYS_PRINT = 1;
static int NEVER_PRINT	= 0;

/* sim control */
extern long		seedval;
extern int		minrequests;
extern int		minwarmup;
extern double		minsimtime;
extern double		warmupproportion;
extern double		simtolerance;
extern double		warmuptolerance;
extern boolean		dumptop;
extern boolean		printtop;
extern boolean		monitorlink;
extern boolean		printversion;
extern char		*mailrecpt;
/* topology */
extern netmodel_t	netmodel;
extern double		netsplit;
extern topologytype_t	toptype;
extern int		edge;
extern int		dimen;
extern int		topsize;
extern int		randgraphmaxdiam;
extern randgraph_t	randgraphmethod;
extern double		randgraphalpha;
extern double		randgraphbeta;
extern double		randgraphplanesize;
extern boolean		randgraphasym;
/* traffic */
extern arrdist_t	arrdist;
extern double		arrshape;
extern double		arrscale;
extern double		trafficmult;
extern arrivalspec_t	arrspec;
extern timeofday_t	timeofday;
extern double		bandwidth;
extern double		spread;
extern holdspec_t	holdspec;
extern double		holdshape;
extern double		holdscale;
extern double		flowtrig;
/* link state updates */
extern double		trigger;
extern double		refreshinterval;
extern double		refreshskew;
extern double		minupdinterval;
/* signalling */
extern blockingpolicy_t	blkpolicy;
/* routing */
extern routepolicy_t	routepolicy;
extern routealg_t	routealg;
extern routealg_t	dualalg;
extern prunepolicy_t	prunep;
extern distfunction_t	distfunc;
extern staleness_t	accuracy;
extern alternate_t	altpolicy;
extern multipolicy_t	multipolicy;
extern uniqpolicy_t	uniqpolicy;
extern boolean		feascheck;
extern int		hopthresh;
extern int		maxhopcount;
extern boolean		reextsigfailure;
extern boolean		prunerecomp;
extern double		conflevel;
extern double		rscostalpha;
extern double		rscostmin;
extern int		rscostlevels;
extern double		rscostconst;
extern double		bwclasssize;
extern double		recompinterval;
extern double		recompskew;
extern boolean		periodicrecomps;
extern boolean		trigrecompsig;
extern boolean		trigrecomproute;
extern boolean		recompafterfail;
extern int		maxsigattempts;

static char routepolicytable[MAX_GEN_TABLE_SIZE][GEN_STR_TABLE_WIDTH];
static char routealgtable[MAX_GEN_TABLE_SIZE][GEN_STR_TABLE_WIDTH];
static char prunepolicytable[MAX_GEN_TABLE_SIZE][GEN_STR_TABLE_WIDTH];
static char distfunctable[MAX_GEN_TABLE_SIZE][GEN_STR_TABLE_WIDTH];
static char toptypetable[MAX_GEN_TABLE_SIZE][GEN_STR_TABLE_WIDTH];
static char accuracytable[MAX_GEN_TABLE_SIZE][GEN_STR_TABLE_WIDTH];
static char arrivaltable[MAX_GEN_TABLE_SIZE][GEN_STR_TABLE_WIDTH];
static char arrdisttable[MAX_GEN_TABLE_SIZE][GEN_STR_TABLE_WIDTH];
static char holdtable[MAX_GEN_TABLE_SIZE][GEN_STR_TABLE_WIDTH];
static char timeofdaytable[MAX_GEN_TABLE_SIZE][GEN_STR_TABLE_WIDTH];
static char altroutetable[MAX_GEN_TABLE_SIZE][GEN_STR_TABLE_WIDTH];
static char randgraphtable[MAX_GEN_TABLE_SIZE][GEN_STR_TABLE_WIDTH];
static char multipolicytable[MAX_GEN_TABLE_SIZE][GEN_STR_TABLE_WIDTH];
static char uniqpolicytable[MAX_GEN_TABLE_SIZE][GEN_STR_TABLE_WIDTH];
static char blockingpolicytable[MAX_GEN_TABLE_SIZE][GEN_STR_TABLE_WIDTH];
static char netmodeltable[MAX_GEN_TABLE_SIZE][GEN_STR_TABLE_WIDTH];

  
void
createOptions ()
{
  strcpy (routepolicytable[ON_DEMAND], "ON_DEMAND");
  strcpy (routepolicytable[PRECOMP], "PRECOMP");
  strcpy (routepolicytable[PRECOMP + 1], "\0");

  strcpy (routealgtable[SPT], "SPT");
  strcpy (routealgtable[WIDE_SHORT], "WIDE_SHORT");

  /* Adding to parse HSF as a routing algorithm option
   * Also adjusted the end accordingly.
   * Ganga - 08/03/2000
   */

  strcpy (routealgtable[HSF], "HSF");
  strcpy (routealgtable[HSF + 1], "\0");
  
  strcpy (prunepolicytable[NO_PRUNE], "NO_PRUNE");
  strcpy (prunepolicytable[DEFAULT_PRUNE], "DEFAULT_PRUNE");
  strcpy (prunepolicytable[DEFAULT_PRUNE + 1], "\0");

  strcpy (distfunctable[HOPCOUNT], "HOPCOUNT");
  strcpy (distfunctable[EXP_SINGLE_COST], "EXP_SINGLE_COST");  
  strcpy (distfunctable[EXP_MINHOP_BIAS], "EXP_MINHOP_BIAS");
  strcpy (distfunctable[WIDEST], "WIDEST");
  strcpy (distfunctable[WIDEST + 1], "\0");

  strcpy (toptypetable[TOP_KNCUBE], "kncube");
  strcpy (toptypetable[TOP_FILE], "file");
  strcpy (toptypetable[TOP_FULL], "full");
  strcpy (toptypetable[TOP_RAND], "rand");
  strcpy (toptypetable[TOP_RAND + 1], "\0");

  strcpy (randgraphtable[RG2], "RG2");
  strcpy (randgraphtable[PURE], "PURE");
  strcpy (randgraphtable[PURE + 1], "\0");

  strcpy (accuracytable[CURRENT], "CURRENT");
  strcpy (accuracytable[STALE], "STALE");
  strcpy (accuracytable[STALE + 1], "\0");

  strcpy (arrivaltable[ARR_UNIFORM], "ARR_UNIFORM");
  strcpy (arrivaltable[ARR_FILE], "ARR_FILE");
  strcpy (arrivaltable[ARR_FILE+1], "\0");

  strcpy (arrdisttable[WEIBULL], "WEIBULL");
  strcpy (arrdisttable[POISSON], "POISSON");
  strcpy (arrdisttable[POISSON+1], "\0");

  strcpy (holdtable[EXP], "EXP");
  strcpy (holdtable[PARETO], "PARETO");
  strcpy (holdtable[USERDEF], "USERDEF");  
  strcpy (holdtable[USERDEF+1], "\0");

  strcpy (timeofdaytable[MORNING], "MORNING");
  strcpy (timeofdaytable[AFTERNOON], "AFTERNOON");
  strcpy (timeofdaytable[EVENING], "EVENING");
  strcpy (timeofdaytable[EVENING+1], "\0");

  strcpy (altroutetable[NO_ALT], "NO_ALT");
  strcpy (altroutetable[ROUND_ROBIN], "ROUND_ROBIN");
  strcpy (altroutetable[STICKY], "STICKY");
  strcpy (altroutetable[STICKY + 1], "\0");

  strcpy (multipolicytable[MULTI], "MULTI");
  strcpy (multipolicytable[FIRST], "FIRST");
  strcpy (multipolicytable[UNIQ], "UNIQ");
  strcpy (multipolicytable[UNIQ + 1], "\0");

  strcpy (uniqpolicytable[MULTI], "SRCHASH");
  strcpy (uniqpolicytable[RANDOM], "RANDOM");
  strcpy (uniqpolicytable[RANDOM + 1], "\0");

  strcpy (blockingpolicytable[NON_BLOCKING], "NON_BLOCKING");
  strcpy (blockingpolicytable[BLOCKING], "BLOCKING");
  strcpy (blockingpolicytable[BLOCKING+1], "\0");

  strcpy (netmodeltable[DUAL], "DUAL");
  strcpy (netmodeltable[SINGLE], "SINGLE");
  strcpy (netmodeltable[SINGLE + 1], "\0");


  insert_option ("-accuracy", "link-state-accuracy", generic_op, (void *) &accuracy,
		 "Link-state to use: CURRENT or STALE", (void *) &ALWAYS_PRINT,
		 (void *) accuracytable);

  insert_option ("-alpha", "link-cost-alpha", double_op, (void *) &rscostalpha,
		 "Link cost function exponent (steepness)", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-altrouting", "alternate-routing", generic_op, (void *) &altpolicy,
		 "Alt. routing policy: ROUND_ROBIN, STICKY, NO_ALT", (void *) &ALWAYS_PRINT,
		 (void *) altroutetable);

  insert_option ("-arrdist", "arrival-dist", generic_op, (void *) &arrdist,
		 "Call arrival distribution: WEIBULL or POISSON", (void *) &ALWAYS_PRINT,
		 (void *) arrdisttable);

  insert_option ("-arrscale", "arrival-scale", double_op, (void *) &arrscale,
		 "Scale parameter for arrival distribution", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-arrshape", "arrival-shape", double_op, (void *) &arrshape,
		 "Shape parameter for arrival distribution", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-arrspec", "arrival-spec", generic_op, (void *) &arrspec,
		 "Traffic matrix: ARR_UNIFORM or ARR_FILE", (void *) &ALWAYS_PRINT,
		 (void *) arrivaltable);

  insert_option ("-bandwidth", "mean-bandwidth", double_op, (void *) &bandwidth,
		 "Mean requested bandwidth", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-blocking", "blocking-policy", generic_op, (void *) &blkpolicy,
		 "Blocking model: NON_BLOCKING or BLOCKING", (void *) &ALWAYS_PRINT,
		 (void *) blockingpolicytable);

  insert_option ("-bwclass", "bw-class-size", double_op, (void *) &bwclasssize,
		 "Base bandwidth class size (for equal class based updates)",
		 (void *) &ALWAYS_PRINT, 0);

  insert_option ("-costconst", "link-cost-constant", double_op, (void *) &rscostconst,
		 "Additive bias for link costs", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-costlevels", "link-cost-levels", int_op, (void *) &rscostlevels,
		 "Number of discrete costs levels", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-dist", "distance-function", generic_op, (void *) &distfunc,
		 "SPT distance policy: EXP_SINGLE_COST, EXP_MINHOP_BIAS, HOPCOUNT", (void *) &ALWAYS_PRINT,
		 (void *) distfunctable);

  insert_option ("-dualalg", "dual-dyn-algorithm", generic_op, (void *) &dualalg,
		 "Dynamic routing algorithm in DUAL netmodel: SPT or WIDE_SHORT",
		 (void *) &ALWAYS_PRINT, (void *) routealgtable);

  insert_option ("-dumptop","dump-topology", boolean_op, (void *) &dumptop,
		 "Dump the topology to a file", (void *) &NEVER_PRINT,
		 0);

  insert_option ("-failrecomp", "recompute-after-fail", boolean_op, (void *) &recompafterfail,
		 "Recompute routes after final signalling failure?", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-feascheck", "feasibility-check", boolean_op, (void *) &feascheck,
		 "Perform feasibility check on extraction?", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-flowtrigger", "flow-trigger", double_op, (void *) &flowtrig,
                 "Long-lived connection detection threshold (time)", (void *) &ALWAYS_PRINT,
                 0);

  insert_option ("-holdspec", "holding-time-spec", generic_op, (void *) &holdspec,
		 "Holding time dist.: EXP, PARETO, USERDEF", (void *) &ALWAYS_PRINT,
		 (void *) holdtable);

  insert_option ("-hopthresh", "hopcount-threshold", int_op, (void *) &hopthresh,
		 "Limiting threshold on non-minimal routes", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-hscale", "holding-time-scale", double_op, (void *) &holdscale,
                 "Scale parm for Pareto holding times or mean for exponential", (void *) &ALWAYS_PRINT,
                 0);

  insert_option ("-hshape", "holding-time-shape", double_op, (void *) &holdshape,
                 "Shape parm for Pareto holding times", (void *) &ALWAYS_PRINT,
                 0);

  insert_option ("-kndim", "kn-dimen", int_op, (void *) &dimen,
		 "Dimensions in k-ary n-cube", (void *) &ALWAYS_PRINT,
		 0);  

  insert_option ("-knedge", "kn-edge", int_op, (void *) &edge,
		 "Edges in k-ary n-cube", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-level", "confidence-level", double_op, (void *) &conflevel,
		 "Confidence level on call blocking rate", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-mail", "mail-recipient", string_op, (void *) &mailrecpt,
		 "Recipient for simulation notifications", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-maxdiam", "max-rand-diameter", int_op, (void *) &randgraphmaxdiam,
		 "Maximum diameter (for random topology)", (void *) &ALWAYS_PRINT,
		 0);
  
  insert_option ("-maxhops", "max-hopcount", int_op, (void *) &maxhopcount,
		 "Maximum hopcount (for Bellman-Ford alg)", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-maxsigs", "max-sig-attempts", int_op, (void *) &maxsigattempts,
		 "Maximum number of signalling attempts per call", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-mincost", "link-cost-minval", double_op, (void *) &rscostmin,
		 "BW corresponding to min link cost value", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-minint", "min-update-interval", double_op, (void *) &minupdinterval,
		 "Minimum interval between link updates", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-mintime", "min-sim-time", double_op, (void *) &minsimtime,
		 "Minimum simulation time", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-monlink","monitor-link", boolean_op, (void *) &monitorlink,
		 "Monitor and log link state over time", (void *) &ALWAYS_PRINT,
		 0);  

  insert_option ("-multipolicy", "multi-route-policy", generic_op, (void *) &multipolicy,
		 "Muliple route policy: MULTI, FIRST, UNIQ", (void *) &ALWAYS_PRINT,
		 (void *) multipolicytable);
  

  insert_option ("-netmodel", "network-model", generic_op, (void *) &netmodel,
		 "Network model: DUAL or SINGLE", (void *) &ALWAYS_PRINT,
		 (void *) netmodeltable);

  insert_option ("-netsplit", "network-split", double_op, (void *) &netsplit,
		 "Network capacity split for short/long traffic", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-pdrecomp", "periodic-recomputations", boolean_op, (void *) &periodicrecomps,
		 "Use periodic route recomputation?", (void *) &ALWAYS_PRINT,
		 0);
  
  insert_option ("-printtop","print-topology-stats", boolean_op, (void *) &printtop,
		 "Print topology stats and exit", (void *) &NEVER_PRINT,
		 0);

  insert_option ("-prune", "prune-policy", generic_op, (void *) &prunep,
		 "Policy: DEFAULT_PRUNE or NO_PRUNE", (void *) &ALWAYS_PRINT,
		 (void *) prunepolicytable);

  insert_option ("-prunerecomp", "prune-on-recomputation", boolean_op, (void *) &prunerecomp,
		 "Prune failed links on recomputation?", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-ralg", "routing-algorithm", generic_op, (void *) &routealg,
		 "Routing algorithm: SPT (dijkstra) or WIDE_SHORT (bellman-ford) or HSF (Ganga-Claypool)",
		 (void *) &ALWAYS_PRINT, (void *) routealgtable);

  insert_option ("-rcskew", "recomputation-skew", double_op, (void *) &recompskew,
		 "Route recomputation period skew", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-recomp", "recomputation-interval", double_op, (void *) &recompinterval,
		 "Route recomputation period", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-reextsig", "reextract-sig-failure", boolean_op, (void *) &reextsigfailure,
		 "Reextract and try again on sig failure?", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-refresh", "refresh-interval", double_op, (void *) &refreshinterval,
		 "Link update refresh period", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-requests", "min-requests", int_op, (void *) &minrequests,
		 "Minimum call requests", (void *) &ALWAYS_PRINT,
		 0);
  
  insert_option ("-rgalpha", "rand-graph-alpha", double_op, (void *) &randgraphalpha,
		 "Random graph parameter for number of edges", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-rgasym", "rand-graph-asymmetric", boolean_op, (void *) &randgraphasym,
		 "Use asymmetric random graphs", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-rgbeta", "rand-graph-beta", double_op, (void *) &randgraphbeta,
		 "Random graph parameter for long-short ratio", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-rgmethod", "rand-graph-method", generic_op, (void *) &randgraphmethod,
		 "Random graph generation method: RG2 (Waxman) or PURE", (void *) &ALWAYS_PRINT,
		 (void *) randgraphtable);

  insert_option ("-rgplane", "rand-graph-planesize", double_op, (void *) &randgraphplanesize,
		 "Random graph parameter for Euclidean plane dimension", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-route", "routing-policy", generic_op, (void *) &routepolicy,
		 "Policy: ON_DEMAND or PRECOMP", (void *) &ALWAYS_PRINT,
		 (void *) routepolicytable);

  insert_option ("-rskew", "refresh-skew", double_op, (void *) &refreshskew,
		 "Link update refresh period skew", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-seedval", "random-seed", long_op, (void *) &seedval,
		 "Random number generator seed", (void *) &ALWAYS_PRINT,
		 0);  

  insert_option ("-simtol", "sim-tolerance", double_op, (void *) &simtolerance,
		 "Simulation confidence tolerance", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-spread", "bandwidth-spread", double_op, (void *) &spread,
		 "Proportional width of bandwidth distribution", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-timeofday", "time-of-day", generic_op, (void *) &timeofday,
		   "Time of day: MORNING, AFTERNOON, EVENING", (void *) &ALWAYS_PRINT,
		 (void *) timeofdaytable);

  insert_option ("-tmultiplier", "traffic-multiplier", double_op, (void *) &trafficmult,
		 "Scaling factor for traffic matrix", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-topology", "topology-type", generic_op, (void *) &toptype,
		 "Type of topology to use: kncube or file or full or rand", (void *) &ALWAYS_PRINT,
		 (void *) toptypetable);

  insert_option ("-topsize", "topology-size", int_op, (void *) &topsize,
		 "Number of nodes in fully-conn. topology", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-trigger", "update-trigger", double_op, (void *) &trigger,
		 "Link update trigger level", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-trigroute", "trigger-recomp-routefail", boolean_op, (void *) &trigrecomproute,
		 "Trigger recomputation on routing failure?", (void *) &ALWAYS_PRINT,
		 0);  

  insert_option ("-trigsig", "trigger-recomp-sigfail", boolean_op, (void *) &trigrecompsig,
		 "Trigger recomputation on signalling failure?", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-uniqpolicy", "uniq-route-policy", generic_op, (void *) &uniqpolicy,
		 "Unique route policy: SRCHASH, RANDOM", (void *) &ALWAYS_PRINT,
		 (void *) uniqpolicytable);

  insert_option ("-version","print-version", boolean_op, (void *) &printversion,
		 "Print the routesim version and exit", (void *) &ALWAYS_PRINT,
		 0);  

  insert_option ("-warmup", "min-warmup", int_op, (void *) &minwarmup,
		 "Minimum warmup call requests", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-warmupprop", "warmup-proportion", double_op, (void *) &warmupproportion,
		 "Min proportion of warmup sim must run", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-warmuptol", "warmup-tolerance", double_op, (void *) &warmuptolerance,
		 "Warmup confidence tolerance", (void *) &ALWAYS_PRINT,
		 0);

  insert_option ("-wsmaxhops", "wide-short-maxhops", int_op, (void *) &maxhopcount,
		 "Maximum hopcount (for WIDE_SHORT alg)", (void *) &ALWAYS_PRINT,
		 0);

}

