/* $Id: rsstat.c,v 3.7 1999/06/02 20:45:01 ashaikh Exp $ */

static const char rcsid[] = "$Id: rsstat.c,v 3.7 1999/06/02 20:45:01 ashaikh Exp $";

#include "rsstat.h"
#include "rsstatlist.h"
#include "stat.h"
#include "rstrace.h"
#include "rstraffic.h"
#include "topology.h"
#include "rsroute.h"
#include "routesim.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

static double  confidence;

extern int warmup;
extern double refreshinterval;

extern int callscreated;
extern int warmupcalls;
extern int actualcalls;
extern double globaltime;
extern double warmuptime;
extern double actualtime;

extern proportionstat_t	    overutilstat;
extern proportionstat_t	    underutilstat;
extern proportionstat_t	    violutilstat;
extern proportionstat_t	    overutilflowstat;
extern proportionstat_t	    underutilflowstat;
extern proportionstat_t	    violutilflowstat;
extern double		    woverutil;
extern double		    wunderutil;
extern double		    wviolutil;
extern int		    reroutedcalls;
extern int		    rerouteattempts;

static int resetAllStats (topology_t *);
static void printRouteCompStats (double, int, int, topology_t *);
static void printPerHopcountStats (topology_t *);
static void printDualPerHopcountStats (topology_t *, topology_t *);
static void printDualRouteCompStats (double, int, int, int, topology_t *, topology_t *);

/********************************************************************************/
int
initRoutesimStats (double conflevel, topology_t *network)
{
  rsstat_t  *stats;
  int	    diameter;

  stats = network->stats;
  diameter = getDiameter (network);
  confidence = conflevel;

  /* init routesim statistics */
  stats->perhcblk = (simpleratio_t *) malloc (sizeof (simpleratio_t) * (diameter + 1));
  stats->perhcrcomps = (simpleratio_t *) malloc (sizeof (simpleratio_t) * (diameter + 1));
  stats->perhcsigatt = (simpleratio_t *) malloc (sizeof (simpleratio_t) * (diameter + 1));
  stats->callsperhop = (simpleratio_t *) malloc (sizeof (simpleratio_t) * (diameter + 1));
  if ( (stats->perhcblk == (simpleratio_t *) 0) ||
       (stats->perhcrcomps == (simpleratio_t *) 0) ||
       (stats->perhcsigatt == (simpleratio_t *) 0) ||
       (stats->callsperhop == (simpleratio_t *) 0) ) {
    RS_PRINT_ERROR ("initRoutesimStats:  memory allocation failure\n");
    return (-1);
  }
  

  resetAllStats (network);
  stats->totalutilsum = stats->utilizationsum = 0.0;
  
  return (0);
}
/********************************************************************************/
int
resetRoutesimStats (topology_t *network)
{
  return (resetAllStats (network));

}
/********************************************************************************/
static int
resetAllStats (topology_t *network)
{
  rsstat_t  *stats;
  int	    i, diameter;

  stats = network->stats;
  diameter = getDiameter (network);

  if (initSimpleRatioStat(&(stats->callblk), confidence) ||
      initSimpleRatioStat(&(stats->updinterval), confidence) ||
      initSimpleRatioStat(&(stats->residual), confidence) ||
      initSimpleRatioStat(&(stats->rcompspersucccall), confidence) ||
      initSimpleRatioStat(&(stats->rcompsperfailcall), confidence) ||
      initSimpleRatioStat(&(stats->sigattperfailcall), confidence) ||
      initSimpleRatioStat(&(stats->sigattpersucccall), confidence) ||
      initSimpleRatioStat(&(stats->routecachehits), confidence) ||
      initSimpleRatioStat(&(stats->rcompinterval), confidence) ||
      initSimpleRatioStat(&(stats->holdtimestat), confidence) ||
      initSimpleRatioStat(&(stats->arrivalratestat), confidence) ||
      initSimpleRatioStat(&(stats->reqbwstat), confidence) ||      
      initSimpleRatioStat(&(stats->extractpersucccall), confidence) ||
      initSimpleRatioStat(&(stats->extractperfailcall), confidence) ||
      initSimpleRatioStat(&(stats->timesincecompsucc), confidence) ||
      initSimpleRatioStat(&(stats->timesincecompfail), confidence) ||
      initSimpleRatioStat(&(stats->timesincelinkupdfail), confidence) ||
      initSimpleRatioStat(&(stats->timesincelinkupdsucc), confidence) ||
      initProportionStat (&(overutilstat), confidence) ||
      initProportionStat (&(underutilstat), confidence) ||
      initProportionStat (&(violutilstat), confidence) ||
      initProportionStat (&(overutilflowstat), confidence) ||
      initProportionStat (&(underutilflowstat), confidence) ||
      initProportionStat (&(violutilflowstat), confidence) ) {
    return (-1);
  }

  /* zero position is unused */
  for (i = 1; i <= diameter; i++) {
    if (initSimpleRatioStat(&(stats->perhcblk[i]), confidence) ||
	initSimpleRatioStat(&(stats->perhcrcomps[i]), confidence) ||
	initSimpleRatioStat(&(stats->perhcsigatt[i]), confidence) ||
	initSimpleRatioStat(&(stats->callsperhop[i]), confidence))
      return (-1);
  }
  
  /* global call stats */
  stats->terminatedcalls = 0;
  stats->sigfailures = 0;
  stats->routefailures = 0;
  stats->sigattempts = 0;
  stats->extractfailures = 0;
  stats->rerouteattempts = 0;
  /* utilizationsum and totalutilsum don't get reset */
  /* update stats */
  stats->rlinkupdates = 0;
  stats->tlinkupdates = 0;
  stats->extraupdates = 0;
  stats->deferredupdates = 0;
  /* traffic stats */
  stats->blockedbw = 0.0;
  stats->blockedhcbw = 0.0;
  stats->requestedhcbw = 0.0;
  stats->blockedhc = 0.0;
  stats->requestedhc = 0.0;
  stats->maxsuccduration = 0.0;  
  /* route stats */
  stats->sumhops = 0;
  stats->summinhops = 0;
  stats->sumhopdiff = 0;
  stats->maxhopdiff = 0;
  stats->longroutes = 0;
  /* these stats aren't associated with a network */
  reroutedcalls = 0;
  rerouteattempts = 0;
  woverutil = wunderutil = wviolutil = 0.0;

  /* Additional stat to measure network cost 
   * in terms of number of nodes visited.
   * Ganga - 03/30/2000
   */
  
  stats->nodesvisited = 0;

  return (0);

}
/********************************************************************************/
void
cleanupRSStats (topology_t *network)
/* free memory allocated for per-hopcount stats
 */     
{
  rsstat_t  *stats;

  stats = network->stats;

  if (stats->perhcblk != (simpleratio_t *) 0)
    free (stats->perhcblk);
  if (stats->perhcrcomps != (simpleratio_t *) 0)
    free (stats->perhcrcomps);
  if (stats->perhcsigatt != (simpleratio_t *) 0)
    free (stats->perhcsigatt);
  if (stats->callsperhop != (simpleratio_t *) 0)
    free (stats->callsperhop);

  stats->perhcblk = stats->perhcrcomps = stats->perhcsigatt = stats->callsperhop = (simpleratio_t *) 0;
}
/********************************************************************************/
void
collectSuccCallStats (call_t *call)
{
  int		hd;
  int		i;
  double	linksum;
  double	minavail, residualstat;
  int		minlink;
  double	cachehit = 0.0;
  topology_t	*network;
  rsstat_t	*stats;

  network = call->network;
  stats = network->stats;
  
  stats->sumhops += (call->routelen - 1);
  stats->summinhops += call->minhops;
  hd = (call->routelen - 1) - call->minhops;
  if (hd > stats->maxhopdiff)
    stats->maxhopdiff  = hd;
  if (hd > 0) {
    stats->longroutes++;
    stats->sumhopdiff += hd;
  }

  linksum = 0.0;
  minavail = getMaxLinkCapacity(network);
  minlink = -1;
  for (i = 0; i < call->routelen - 1; i++) {
    linksum += (call->bandw / call->links[i]->capacity) * call->length;
    if (call->links[i]->availbw.current < minavail) {	/* find bottleneck residual link */
      minavail = call->links[i]->availbw.current;
      minlink = i;
    }
  }
  residualstat = (minavail / call->links[minlink]->capacity) /
    (1.0 - (minavail / call->links[minlink]->capacity));
  if (updateSimpleRatioStat (residualstat, &stats->residual)) {
    RS_PRINT_ERROR ("collectSuccCallStats: Error updating residual\n");
  }
  stats->utilizationsum +=  linksum;

  if (call->length > stats->maxsuccduration) {
    stats->maxsuccduration = call->length;
    stats->maxdurationtime = call->gentime;
  }

  if (updateSimpleRatioStat (0.0, &stats->callblk)) {
    RS_PRINT_ERROR ("collectSuccCallStats: Error updating callblk\n");
  }
  if (updateSimpleRatioStat (0.0, &stats->perhcblk[call->minhops])) {
    RS_PRINT_ERROR ("collectSuccCallStats: Error updating perchcblk\n");
  }
  if (updateSimpleRatioStat (call->routecomps, &stats->perhcrcomps[call->minhops])) {
    RS_PRINT_ERROR ("collectSuccCallStats: Error updating perhcrcomps\n");
  }
  if (updateSimpleRatioStat (call->sigattempts, &stats->perhcsigatt[call->minhops])) {
    RS_PRINT_ERROR ("collectSuccCallStats: Error updating perhcsigatt\n");
  }  
  if (updateSimpleRatioStat (call->routecomps, &stats->rcompspersucccall)) {
    RS_PRINT_ERROR ("collectSuccCallStats: Error updating rcompspersucccall\n");
  }
  if (updateSimpleRatioStat ((call->sigattempts), &stats->sigattpersucccall)) {
    RS_PRINT_ERROR ("collectSuccCallStats: Error updating sigattpersucccall\n");
  }
  if (updateSimpleRatioStat (call->extractions, &stats->extractpersucccall)) {
    RS_PRINT_ERROR ("collectSuccCallStats: Error updating extractpercall\n");
  }

  if ((call->routecomps == 0) && (call->sigattempts == 1))
    cachehit = 1.0; /* cache hit with no recomputations and one signalling attempts */
    
  if (updateSimpleRatioStat (cachehit, &stats->routecachehits)) {
    RS_PRINT_ERROR ("collectSuccCallStats: Error updating routecachhits\n");
  }

  if (updateSimpleRatioStat ((call->gentime - getLastRouteCompTime(call->source, network)),
			     &stats->timesincecompsucc)) {
    RS_PRINT_ERROR ("collectSuccCallStats: Error updating timesincecompsucc\n");
  }
  if (updateSimpleRatioStat ((call->gentime - (floor(call->gentime/refreshinterval)*refreshinterval)),
			     &stats->timesincelinkupdsucc)) {
    RS_PRINT_ERROR ("collectSuccCallStats: Error updating timesincelinkupdsucc\n");
  }

#if RS_TRACE_LEVEL >= TRACE_DETAILS
  printCall (call);
#endif

}
/********************************************************************************/
void
collectFailCallStats (call_t *call)
{
  topology_t	*network;
  rsstat_t	*stats;

  network = call->network;
  stats = network->stats;
  
  stats->blockedbw += call->bandw;
  stats->blockedhcbw += call->bandw * call->minhops;
  stats->blockedhc += call->minhops;

  if (updateSimpleRatioStat (1.0, &stats->callblk)) {
    RS_PRINT_ERROR ("collectFailCallStats: Error updating callblk\n");
  }
  if (updateSimpleRatioStat (1.0, &stats->perhcblk[call->minhops])) {
    RS_PRINT_ERROR ("collectFailCallStats: Error updating perhcblk\n");
  }
  if (updateSimpleRatioStat (call->routecomps, &stats->perhcrcomps[call->minhops])) {
    RS_PRINT_ERROR ("collectFailCallStats: Error updating perhcrcomps\n");
  }
  if (updateSimpleRatioStat (call->sigattempts, &stats->perhcsigatt[call->minhops])) {
    RS_PRINT_ERROR ("collectFailCallStats: Error updating perhcsigatt\n");
  }
  if (updateSimpleRatioStat (call->routecomps, &stats->rcompsperfailcall)) {
    RS_PRINT_ERROR ("collectFailCallStats: Error updating rcompsperfailcall\n");
  }
  if (updateSimpleRatioStat ((call->sigattempts), &stats->sigattperfailcall)) {
    RS_PRINT_ERROR ("collectFailCallStats: Error updating sigattperfailcall\n");
  }
  if (updateSimpleRatioStat (call->extractions, &stats->extractperfailcall)) {
    RS_PRINT_ERROR ("collectFailCallStats: Error updating extractpercall\n");
  }
  if (updateSimpleRatioStat (0.0, &stats->routecachehits)) {
    RS_PRINT_ERROR ("collectFailCallStats: Error updating routecachhits\n");
  }
  if (updateSimpleRatioStat ((call->gentime - getLastRouteCompTime(call->source, network)),
			     &stats->timesincecompfail)) {
    RS_PRINT_ERROR ("collectSuccCallStats: Error updating timesincecompfail\n");
  }

  if (updateSimpleRatioStat ((call->gentime - (floor(call->gentime/refreshinterval)*refreshinterval)),
			     &stats->timesincelinkupdfail)) {
    RS_PRINT_ERROR ("collectSuccCallStats: Error updating timesincelinkupdfail\n");
  }
  
  
}
/********************************************************************************/
void
collectReqCallStats (call_t *call)
{
  topology_t	*network;
  rsstat_t	*stats;

  network = call->network;
  stats = network->stats;
  
  stats->requestedhcbw += call->bandw * call->minhops;
  stats->requestedhc += call->minhops;
  stats->totalutilsum += (call->bandw * call->minhops * call->length);

  if (updateSimpleRatioStat (call->bandw, &stats->reqbwstat)) {
    RS_PRINT_ERROR ("collectReqCallStats: Error updating reqbwstat\n");
  }

  if (call->minhops > getDiameter(network)) {
    RS_PRINT_ERROR1 ("minhops %d out of range\n", call->minhops);
  }
  else {
    if (updateSimpleRatioStat (1.0, &stats->callsperhop[call->minhops])) {
      RS_PRINT_ERROR ("collectReqCallStats: Error updating callsperhop\n");
    }
  }

  if (updateSimpleRatioStat (call->length, &stats->holdtimestat)) {
    RS_PRINT_ERROR ("collectReqCallStats: Error updating holdtimestat\n");
  }

   

#if RS_TRACE_LEVEL >= TRACE_ALL
    calltable[call->source][call->dest] += 1;
#endif
}
/********************************************************************************/
static void
printPerHopcountStats (topology_t *network)
{
  int	    i;
  int	    diameter;
  rsstat_t  *stats;
  

  diameter = getDiameter(network);
  stats = network->stats;

  printf ("%-20s\n", "Hopcount Route comp. breakdown:");
  for (i = 1; i <= diameter; i++) {
    if (stats->perhcblk[i].denomsum > 0)
      printf ("  %d%-24s\t%.4f\n", i, "-hop Computation rate:", stats->perhcrcomps[i].currentrate);
  }  
  printf ("%-20s\n", "Hopcount Sig. attempt breakdown:");
  for (i = 1; i <= diameter; i++) {
    if (stats->perhcblk[i].denomsum > 0)
      printf ("  %d%-24s\t%.4f\n", i, "-hop Signalling rate:", stats->perhcsigatt[i].currentrate);      
  }  
  printf ("%-20s\n", "Hopcount blocking breakdown:");
  for (i = 1; i <= diameter; i++) {
    if (stats->perhcblk[i].denomsum > 0)
      printf ("  %d%-24s\t%.4f\n", i, "-hop Blocking rate:", stats->perhcblk[i].currentrate);      
  }
}
/********************************************************************************/
static void
printRouteCompStats (double time, int totalcalls, int totalcomps, topology_t *network)
{
  int	    numnodes;
  rsstat_t  *stats;

  numnodes = getNumNodes(network);
  stats = network->stats;
  
  printf ("%-25s\n", "Route computation stats:");
  printf ("  %-25s\t%.4f\n", "Overall computation rate:", (double)totalcomps / (double)numnodes / time);  
  printf ("  %-25s\t%.4f\n", "Avg. route comps per node:", (double)totalcomps / numnodes);
  printf ("  %-25s\t%.4f\n", "Route cache hit rate:", stats->routecachehits.currentrate);  
  printf ("  %-25s\t%.4f\n", "Route comp per admitted:", stats->rcompspersucccall.currentrate);
  printf ("  %-25s\t%.4f\n", "Route comp per blocked:", stats->rcompsperfailcall.currentrate);
  printf ("  %-25s\t%.4f\n", "Avg. recomp interval:", stats->rcompinterval.currentrate);
  printf ("  %-25s\t%.4f\n", "Time since recomp (blocked):", stats->timesincecompfail.currentrate);
  printf ("  %-25s\t%.4f\n", "Time since recomp (admitted):", stats->timesincecompsucc.currentrate);
  printf ("  %-25s\t%.3f\n", "Extract attempts:", (stats->extractpersucccall.numersum + stats->extractperfailcall.numersum));
  printf ("  %-25s\t%.3f\n", "Extract att. per call:", 
	  (stats->extractpersucccall.numersum + stats->extractperfailcall.numersum)/(double)totalcalls);  
  printf ("  %-25s\t%.3f\n", "Extract att. per admitted:", stats->extractpersucccall.currentrate);
  printf ("  %-25s\t%.3f\n", "Extract att. per blocked:", stats->extractperfailcall.currentrate);
  printf ("  %-25s\t%d\n", "Extraction failures:", stats->extractfailures);

}
/********************************************************************************/
void
printSingleStats (topology_t *network)
{

  int	    i, numnodes, numlinks;
  int	    totalcalls, totalcomps = 0;
  double    bwrejexp, hcrejexp, bwhcrejexp = 0.0;
  double    usedtime, timeinterval;
  double    measuredload, offeredload, avghopcount;
  rsstat_t  *stats;

  stats = network->stats;
  if (warmup) {
    totalcalls = warmupcalls;
    timeinterval = warmuptime;
    usedtime = warmuptime;
  }
  else {
    totalcalls = actualcalls;
    usedtime = actualtime;
    timeinterval = usedtime - warmuptime;
  }

  numnodes = getNumNodes (network);
  numlinks = getNumLinks (network);
  /* calculate load statistics */
  avghopcount = getAvgHopcount (network);
  offeredload = (avghopcount / getAvgInterarrival(-1, -1) * getAvgBandwidth() / getAvgLinkCapacity(network) *
		 getAvgHoldTime() * numnodes / ((double)numlinks));
  measuredload = stats->totalutilsum / globaltime / getTotalLinkCapacity(network);

  /* route computations over all nodes */
  for (i = 0; i < numnodes; i++) {
    totalcomps += getNumRouteComps(i, network);
  }

  printf ("---------------------\n");
  printf ("%-20s\t%.3f\n", "Total elapsed time:", globaltime);
  printf ("%-20s\t%d\n", "Total calls created:", totalcalls);
  printf ("%-20s\t%d\n", "Warmup calls created:", warmupcalls);
  printf ("%-20s\t%.3f\n", "Warmup time:", warmuptime);  
  printf ("%-20s\t%d\n", "Terminated calls:", stats->terminatedcalls);
  printf ("%-20s\t%d\n", "Failed calls:", stats->routefailures + stats->sigfailures);
  printf ("  %-25s\t%d\n", "Routing failures:", stats->routefailures);

  printf ("  %-25s\t%.3f\n", "Route failure rate:", (double)stats->routefailures/timeinterval);
  printf ("  %-25s\t%d\n", "Signalling failures:", stats->sigfailures);
  printf ("  %-25s\t%.3f\n", "Sig. failure rate:", (double)stats->sigfailures/timeinterval);  
  printf ("%-20s\n", "Blocking stats:");
  printf ("  %-25s\t", "Blocking policy:");
  printBlockingPolicy(network);
  printf ("\n");  
  printf ("  %-25s\t%.4f\n", "Call blocking rate:", stats->callblk.currentrate);
  printf ("  %-25s\t%.4f\n", "Call blocking std. dev.:", sqrt(stats->callblk.currentvar));
  printf ("  %.1f%% %-25s\t[%.4f, %.4f]\n", confidence, "Call blk rate interval:",
	  stats->callblk.conf.lb, stats->callblk.conf.ub);
  printf ("  %-25s\t%.4f\n", "BW blocking rate:", stats->blockedbw/stats->reqbwstat.numersum);
  printf ("  %-25s\t%.4f\n", "Hopcount blocking rate:", stats->blockedhc/stats->requestedhc);
  printf ("  %-25s\t%.4f\n", "Signalling blocking:",
	  (double)stats->sigfailures / (double)totalcalls);
  printf ("  %-25s\t%.4f\n", "Routing blocking:",
	  (double)stats->routefailures / (double)totalcalls);	  
  printf ("  %-25s\t%.4f\n", "Signalling blocking prop.:",
	  (double)stats->sigfailures / (double)(stats->sigfailures + stats->routefailures));
  printf ("  %-25s\t%.4f\n", "Routing blocking prop.:",
	  (double)stats->routefailures / (double)(stats->sigfailures + stats->routefailures));  
  printf ("  %-25s\t%.4f\n", "Hopcount-BW blocking rate:", stats->blockedhcbw/stats->requestedhcbw);
  bwrejexp = (stats->blockedbw/stats->reqbwstat.numersum) / ((float) (stats->sigfailures + stats->routefailures)  / (float) totalcalls) * getAvgBandwidth();
  hcrejexp = (stats->blockedhc/stats->requestedhc) / ((float) (stats->sigfailures + stats->routefailures) /
					(float) totalcalls) * avghopcount;
  bwhcrejexp = (stats->blockedhcbw/stats->requestedhcbw) / ((float) (stats->sigfailures + stats->routefailures)  / (float) totalcalls) * getAvgBandwidth() * avghopcount;
  printf ("  %-25s\t%.3f\n", "Hops-BW blocking covariance:", bwhcrejexp -
	  (bwrejexp * hcrejexp));
  printf ("%-20s\t%.3f\n", "Total injected load:", offeredload);
  printf ("%-20s\t%.3f\n", "Total measured load:", measuredload);
  printf ("%-20s\t%.3f\n", "Approx. utilization:", (1 - (stats->blockedhcbw/stats->requestedhcbw)) *
	  (offeredload));
  printf ("%-20s\t%.3f\n", "Exact utilization:", stats->utilizationsum / (globaltime * ((double)numlinks)));
  printf ("%-20s\t%.3f\n", "Carried load:", offeredload * (1.0 - stats->callblk.currentrate));
  printRouteCompStats(timeinterval, totalcalls, totalcomps, network);
  printf ("  %-25s\t", "Routing policy:");
  printRoutingPolicy();
  printf ("\n");
  printf ("  %-25s\t", "Routing algorithm:");
  printRoutingAlgorithm(network);
  printf ("\n");
  printf ("  %-25s\t", "Alternate routing policy:");
  printAltPolicy(network);
  printf ("\n");
  printf ("%-25s\n", "Signalling stats:");
  printf ("  %-25s\t%.4f\n", "Sig. att. per requested call:",
	  (double)stats->sigattempts / (double)totalcalls);  
  printf ("  %-25s\t%.4f\n", "Sig. att. per signalled call:",
	  (double)stats->sigattempts / (double)(totalcalls - stats->routefailures));
  printf ("  %-25s\t%.4f\n", "Signalling rate per node:",
	  (double)stats->sigattempts / timeinterval / (double)numnodes);
  printf ("  %-25s\t%.4f\n", "Sig. attempts per admitted:", stats->sigattpersucccall.currentrate);
  printf ("  %-25s\t%.4f\n", "Sig. attempts per blocked:", stats->sigattperfailcall.currentrate);  
  printf ("  %-25s\t", "Route pruning policy:");
  printPrunePolicy(network);
  printf ("\n");
  printf ("  %-25s\t", "Distance function:");
  printDistFunction(network);
  printf ("\n");
  printf ("  %-25s\t", "Link-state accuracy:");
  printStaleness();
  printf ("\n");
  printf ("%-20s\n", "Route stats:");    
  printf ("  %-25s\t%.3f\n", "Hopcount to minhop ratio:", (float) stats->sumhops / (float) stats->summinhops);
  printf ("  %-25s\t%.3f\n", "Avg. hopcount (success):", (float) stats->sumhops / (float) stats->terminatedcalls);
  printf ("  %-25s\t%.3f\n", "Avg. hopcount (total):", (float) stats->requestedhc / (float)totalcalls);  
  printf ("  %-25s\t%.3f\n", "Calc. avg. hopcount:", avghopcount);
  printf ("  %-25s\t%.3f\n", "Avg. minhopcount:", (float) stats->summinhops / (float) totalcalls);  
  printf ("  %-25s\t%d\n", "Num. long routes:", stats->longroutes);
  printf ("  %-25s\t%.3f\n", "Avg. hopcount diff:",
	  stats->longroutes == 0 ? 0 : ((float) stats->sumhopdiff / (float) stats->longroutes));
  printf ("  %-25s\t%d\n", "Max hopcount diff:", stats->maxhopdiff);
  printPerHopcountStats(network);   
  printf ("%-20s\n", "Link update stats:");
  printf ("  %-25s\t%.3f\n", "Link update rate:",
	  (float) (stats->rlinkupdates + stats->tlinkupdates) / timeinterval);
  printf ("  %-25s\t%.3f\n", "Per-link update rate:",
	  (double) (stats->rlinkupdates + stats->tlinkupdates) / (timeinterval *
						    (double)numlinks));  
  printf ("  %-25s\t%.4f\n", "Avg. update interval:", stats->updinterval.currentrate);
  printf ("  %.1f%% %-25s\t[%.4f, %.4f]\n", confidence, "Upd. interval spread:",
	  stats->updinterval.conf.lb, stats->updinterval.conf.ub);
  printf ("  %-25s\t%d\n", "Total link updates:", stats->rlinkupdates + stats->tlinkupdates);
  printf ("  %-25s\t%d\n", "Refresh link updates:", stats->rlinkupdates);
  printf ("  %-25s\t%d\n", "Triggered link updates:", stats->tlinkupdates);
  printf ("  %-25s\t%d\n", "Deferred link updates:", stats->deferredupdates);
  printf ("  %-25s\t%d\n", "Extraneous updates:", stats->extraupdates);
  printf ("  %-25s\t%.4f\n", "Time since update (blocked):", stats->timesincelinkupdfail.currentrate);
  printf ("  %-25s\t%.4f\n", "Time since update (admitted):", stats->timesincelinkupdsucc.currentrate);  
  printf ("%-20s\n", "Traffic stats:");
  printf ("  %-25s\t%.4f\n", "Avg. residual bandwidth:", stats->residual.currentrate);
  printf ("  %.1f%% %-25s\t[%.4f, %.4f]\n", confidence, "Residual bw spread:",
	  stats->residual.conf.lb, stats->residual.conf.ub);
  printf ("  %-25s\t%.3f\n", "Maximumum requested bw:", stats->reqbwstat.max);
  printf ("  %-25s\t%.3f\n", "Minimum requested bw:", stats->reqbwstat.min);
  printf ("  %-25s\t", "Arrival rate spec:");
  printArrivalSpec ();
  printf ("\n");    
  printf ("  %-25s\t%.3f\n", "Max. interarrival time:", stats->arrivalratestat.max);
  printf ("  %-25s\t%.3f\n", "Min. interarrival time:", stats->arrivalratestat.min);
  printf ("  %-25s\t%.3f\n", "Avg. interarrival time:", stats->arrivalratestat.currentrate);  
  printf ("  %-25s\t%.3f\n", "Avg. arrival rate:", 1.0/stats->arrivalratestat.currentrate);  
  printf ("  %-25s\t", "Holding time spec:");
  printHoldSpec ();
  printf ("\n");
  printf ("  %-25s\t%.3f\n", "Avg. req. holding time:", stats->holdtimestat.currentrate);  
  printf ("  %-25s\t%.3f\n", "Max req. holding time:", stats->holdtimestat.max);
  printf ("  %-25s\t%.3f\n", "Min req. holding time:", stats->holdtimestat.min);
  printf ("  %-25s\t%.3f (%.3f)\n", "Max succ. holding time:", stats->maxsuccduration, stats->maxdurationtime);
  printf ("  %-25s\t%.3f\n", "Holding time variance:", stats->holdtimestat.currentvar);
  printf ("%-20s\n", "Link stats:");
  printLinkUtilStats(network);
  printf ("  %-25s\t%.4f\n", "Time underutilized:", underutilstat.proportion);
  printf ("  %-25s\t%.4f\n", "Time-flows underutilized:", underutilflowstat.proportion);  
  printf ("  %-25s\t%.4f\n", "Avg. underutilization:", wunderutil / underutilstat.numersum);
  printf ("  %-25s\t%.4f\n", "Time overutilized:", overutilstat.proportion);
  printf ("  %-25s\t%.4f\n", "Time-flows overutilized:", overutilflowstat.proportion);  
  printf ("  %-25s\t%.4f\n", "Avg. overutilization:", woverutil / overutilstat.numersum);  
  printf ("  %-25s\t%.4f\n", "Time in violation:", violutilstat.proportion);
  printf ("  %-25s\t%.4f\n", "Time-flows in violation:", violutilflowstat.proportion);
  printf ("  %-25s\t%.4f\n", "Avg. violation:", wviolutil / violutilstat.numersum);
  printf ("  %-25s\t%.3f\n", "Total link util stat time:",
	  (underutilstat.numersum + overutilstat.numersum + violutilstat.numersum));
  printCallsPerLinkStats (network);
  printf ("  %-25s\t%d\n", "Total num links:", numlinks);

  /* Printing the network cost stats
   * Ganga - 03/30/2000
   */

  printf (" \n%-25s\t%d\n\n", "Total nodes visited: ", stats->nodesvisited );


#if RS_TRACE_LEVEL >= TRACE_ALL
  printCallTable (totalcalls);
#endif
}
/********************************************************************************/
static void
printDualPerHopcountStats (topology_t *net1, topology_t *net2)
{
  int	    i;
  int	    diameter;
  rsstat_t  *stats1, *stats2;
  

  diameter = getDiameter(net1);
  stats1 = net1->stats;
  stats2 = net2->stats;  

  printf ("%-20s\n", "Hopcount Route comp. breakdown:");
  for (i = 1; i <= diameter; i++) {
    if ( (stats1->perhcblk[i].denomsum > 0) ||
	 (stats2->perhcblk[i].denomsum > 0) )
      printf ("  %d%-24s\t\t%.4f\t%.4f\n", i, "-hop Computation rate:",
	      stats1->perhcrcomps[i].currentrate,
	      stats2->perhcrcomps[i].currentrate);
  }  
  printf ("%-20s\n", "Hopcount Sig. attempt breakdown:");
  for (i = 1; i <= diameter; i++) {
    if ( (stats1->perhcblk[i].denomsum > 0) ||
	 (stats2->perhcblk[i].denomsum > 0) )	 
      printf ("  %d%-24s\t\t%.4f\t%.4f\n", i, "-hop Signalling rate:",
	      stats1->perhcsigatt[i].currentrate,
	      stats2->perhcsigatt[i].currentrate);      
  }  
  printf ("%-20s\n", "Hopcount blocking breakdown:");
  for (i = 1; i <= diameter; i++) {
    if ( (stats1->perhcblk[i].denomsum > 0) ||
	 (stats2->perhcblk[i].denomsum > 0) )
      printf ("  %d%-24s\t\t%.4f\t%.4f\n", i, "-hop Blocking rate:",
	      stats1->perhcblk[i].currentrate,
	      stats2->perhcblk[i].currentrate);
  }
}
/********************************************************************************/
static void
printDualRouteCompStats (double time, int totalcalls, int totalcomps1, int totalcomps2, topology_t *net1, topology_t *net2)
{
  int	    numnodes;
  double    comprate1, comprate2;
  rsstat_t  *stats1, *stats2;

  numnodes = getNumNodes(net1);
  stats1 = net1->stats;
  stats2 = net2->stats;

  comprate1 = (double)totalcomps1 / (double)numnodes / time;
  comprate2 = (double)totalcomps2 / (double)numnodes / time;
  
  printf ("%-25s\n", "Route computation stats:");
  printf ("  %-25s\t%.4f\t\t%.4f\n", "Overall computation rate:", comprate1, comprate2);
  printf ("  %-25s\t%.4f\t\t%.4f\n", "Avg. route comps per node:",
	  comprate1*time, comprate2*time);
  printf ("  %-25s\t%.4f\t\t%.4f\n", "Route cache hit rate:",
	  stats1->routecachehits.currentrate, stats2->routecachehits.currentrate);  
  printf ("  %-25s\t%.4f\t\t%.4f\n", "Route comp per admitted:",
	  stats1->rcompspersucccall.currentrate, stats2->rcompspersucccall.currentrate);
  printf ("  %-25s\t%.4f\t\t%.4f\n", "Route comp per blocked:",
	  stats1->rcompsperfailcall.currentrate, stats2->rcompsperfailcall.currentrate);
  printf ("  %-25s\t%.4f\t\t%.4f\n", "Avg. recomp interval:",
	  stats1->rcompinterval.currentrate, stats2->rcompinterval.currentrate);
  printf ("  %-25s\t%.4f\t%.4f\n", "Time since recomp (blocked):",
	  stats1->timesincecompfail.currentrate, stats2->timesincecompfail.currentrate);
  printf ("  %-25s\t%.4f\t%.4f\n", "Time since recomp (admitted):",
	  stats1->timesincecompsucc.currentrate, stats2->timesincecompfail.currentrate);
  printf ("  %-25s\t%.3f\t%.3f\n", "Extract attempts:",
	  (stats1->extractpersucccall.numersum + stats1->extractperfailcall.numersum),
	  (stats2->extractpersucccall.numersum + stats2->extractperfailcall.numersum));
  printf ("  %-25s\t%.3f\t\t%.3f\n", "Extract att. per call:", 
	  (stats1->extractpersucccall.numersum + stats1->extractperfailcall.numersum)/(double)totalcalls,
	  (stats2->extractpersucccall.numersum + stats2->extractperfailcall.numersum)/(double)totalcalls);  
  printf ("  %-25s\t%.3f\t\t%.3f\n", "Extract att. per admitted:",
	  stats1->extractpersucccall.currentrate, stats2->extractpersucccall.currentrate);
  printf ("  %-25s\t%.3f\t\t%.3f\n", "Extract att. per blocked:",
	  stats1->extractperfailcall.currentrate, stats2->extractperfailcall.currentrate);
  printf ("  %-25s\t%d\t\t%d\n", "Extraction failures:",
	  stats1->extractfailures, stats2->extractfailures);

}
/********************************************************************************/
void
printDualStats (topology_t *net, topology_t *net1, topology_t *net2)
{

  int	    i, numnodes, numlinks;
  int	    totalcalls, totalcomps1, totalcomps2 = 0;
  double    bwrejexp, hcrejexp, bwhcrejexp = 0.0;
  double    usedtime, timeinterval;
  double    offeredload, avghopcount;
  double    measuredload1, measuredload2;
  rsstat_t  *stats1, *stats2;

  stats1 = net1->stats;
  stats2 = net2->stats;

  if (warmup) {
    totalcalls = warmupcalls;
    timeinterval = warmuptime;
    usedtime = warmuptime;
  }
  else {
    totalcalls = actualcalls;
    usedtime = actualtime;
    timeinterval = usedtime - warmuptime;
  }

  numnodes = getNumNodes (net);
  numlinks = getNumLinks (net);
  /* calculate load statistics */
  avghopcount = getAvgHopcount (net);
  offeredload = (avghopcount / getAvgInterarrival(-1, -1) * getAvgBandwidth() /
			 getAvgLinkCapacity(net) * getAvgHoldTime() * numnodes / ((double)numlinks));
  measuredload1 = stats1->totalutilsum / globaltime / getTotalLinkCapacity(net1);
  measuredload2 = stats2->totalutilsum / globaltime / getTotalLinkCapacity(net2);  

  /* route computations over all nodes */
  for (i = 0; i < numnodes; i++) {
    totalcomps1 = getNumRouteComps(i, net1);
    totalcomps2 = getNumRouteComps(i, net2);
  }

  printf ("---------------------\n");
  printf ("  %-25s\t%s\t%s\n", " ", "Network 1", "Network 2");
  printf ("%-20s\t%.3f\n", "Total elapsed time:", globaltime);
  printf ("%-20s\t%d\n", "Total calls created:", totalcalls);
  printf ("%-20s\t%d\n", "Warmup calls created:", warmupcalls);
  printf ("%-20s\t%.3f\n", "Warmup time:", warmuptime);  
  printf ("%-20s\t%d\t\t%d\n", "Terminated calls:", stats1->terminatedcalls, stats2->terminatedcalls);
  printf ("%-20s\t%d\t\t%d\n", "Failed calls:", stats1->routefailures + stats1->sigfailures, stats2->routefailures + stats2->sigfailures);
  printf ("  %-25s\t%d\t\t%d\n", "Routing failures:", stats1->routefailures, stats2->routefailures);
  printf ("  %-25s\t%.3f\t\t%.3f\n", "Route failure rate:", (double)stats1->routefailures/timeinterval, (double)stats2->routefailures/timeinterval);
  printf ("  %-25s\t%d\t\t%d\n", "Signalling failures:", stats1->sigfailures, stats2->sigfailures);
  printf ("  %-25s\t%.3f\t\t%.3f\n", "Sig. failure rate:", (double)stats1->sigfailures/timeinterval, (double)stats2->sigfailures/timeinterval);
  printf ("%-20s\n", "Reroute stats:");
  printf ("  %-25s\t%d\n", "Reroute attempts:", rerouteattempts);
  printf ("  %-25s\t%d\n", "Successful reroutes:", reroutedcalls);
  printf ("%-20s\n", "Blocking stats:");
  printf ("  %-25s\t", "Blocking policy:");
  printBlockingPolicy(net1);
  printf ("\t");
  printBlockingPolicy(net2);
  printf ("\n");  
  printf ("  %-25s\t%.4f\t\t%.4f\n", "Call blocking rate:", stats1->callblk.currentrate,
	  stats2->callblk.currentrate);
  printf ("  %-25s\t%.4f\t\t%.4f\n", "Call blocking std. dev.:", sqrt(stats1->callblk.currentvar), sqrt(stats2->callblk.currentvar));
  printf ("  %.1f%% %-25s\t[%.4f, %.4f]\n", confidence, "Call blk rate interval:",
	  stats1->callblk.conf.lb, stats1->callblk.conf.ub);
  printf ("  %-25s\t%.4f\t\t%.4f\n", "BW blocking rate:", stats1->blockedbw/stats1->reqbwstat.numersum, stats2->blockedbw/stats2->reqbwstat.numersum);
  printf ("  %-25s\t%.4f\t\t%.4f\n", "Hopcount blocking rate:", stats1->blockedhc/stats1->requestedhc, stats2->blockedhc/stats2->requestedhc);
  printf ("  %-25s\t%.4f\t\t%.4f\n", "Signalling blocking:",
	  (double)stats1->sigfailures / (double)totalcalls, (double)stats2->sigfailures / (double)totalcalls);
  printf ("  %-25s\t%.4f\t\t%.4f\n", "Routing blocking:",
	  (double)stats1->routefailures / (double)totalcalls, (double)stats2->routefailures / (double)totalcalls);	  
  printf ("  %-25s\t%.4f\t\t%.4f\n", "Signalling blocking prop.:",
	  (double)stats1->sigfailures / (double)(stats1->sigfailures + stats1->routefailures),
	  (double)stats2->sigfailures / (double)(stats2->sigfailures + stats2->routefailures));
  printf ("  %-25s\t%.4f\t\t%.4f\n", "Routing blocking prop.:",
	  (double)stats1->routefailures / (double)(stats1->sigfailures + stats1->routefailures),
	  (double)stats2->routefailures / (double)(stats2->sigfailures + stats2->routefailures));  
  printf ("  %-25s\t%.4f\t\t%.4f\n", "Hopcount-BW blocking rate:",
	  stats1->blockedhcbw/stats1->requestedhcbw, stats2->blockedhcbw/stats2->requestedhcbw);
  bwrejexp = (stats1->blockedbw/stats1->reqbwstat.numersum) / ((float) (stats1->sigfailures + stats1->routefailures)  / (float) totalcalls) * getAvgBandwidth();
  hcrejexp = (stats1->blockedhc/stats1->requestedhc) / ((float) (stats1->sigfailures + stats1->routefailures) / (float) totalcalls) * avghopcount;
  bwhcrejexp = (stats1->blockedhcbw/stats1->requestedhcbw) / ((float) (stats1->sigfailures + stats1->routefailures)  / (float) totalcalls) * getAvgBandwidth() * avghopcount;
  printf ("  %-25s\t%.3f\n", "Hops-BW blocking covariance:", bwhcrejexp -
	  (bwrejexp * hcrejexp));
  printf ("%-20s\t%.3f\n", "Total injected load:", offeredload);
  printf ("%-20s\t%.3f\n", "Total measured load:", measuredload2);
  printf ("%-20s\t%.3f\t%.3f\n", "Approx. utilization:",
	  (1 - (stats1->blockedhcbw/stats1->requestedhcbw)) * (offeredload),
	  (1 - (stats2->blockedhcbw/stats2->requestedhcbw)) * (offeredload));
  printf ("%-20s\t%.3f\t%.3f\n", "Exact utilization:",
	  stats1->utilizationsum / (globaltime * ((double)numlinks)),
	  stats2->utilizationsum / (globaltime * ((double)numlinks)));
  printf ("%-20s\t%.3f\t%.3f\n", "Carried load:",
	  offeredload * (1.0 - stats1->callblk.currentrate),
	  offeredload * (1.0 - stats2->callblk.currentrate));
  printf ("  %-25s\t", "Routing policy:");
  printRoutingPolicy();
  printf ("\n");
  printf ("  %-25s\t", "Routing algorithm:");
  printRoutingAlgorithm(net1);
  printf ("\t");
  printRoutingAlgorithm(net2);  
  printf ("\n");
  printf ("  %-25s\t", "Alternate routing policy:");
  printAltPolicy(net1);
  printf ("\t");
  printAltPolicy(net2);    
  printf ("\n");  
  printDualRouteCompStats(timeinterval, totalcalls, totalcomps1, totalcomps2, net1, net2);
  printf ("%-25s\n", "Signalling stats:");
  printf ("  %-25s\t%.4f\t\t%.4f\n", "Sig. att. per requested call:",
	  (double)stats1->sigattempts / (double)totalcalls,
	  (double)stats2->sigattempts / (double)totalcalls);  
  printf ("  %-25s\t%.4f\t\t%.4f\n", "Sig. att. per signalled call:",
	  (double)stats1->sigattempts / (double)(totalcalls - stats1->routefailures),
	  (double)stats2->sigattempts / (double)(totalcalls - stats2->routefailures));
  printf ("  %-25s\t%.4f\t\t%.4f\n", "Signalling rate per node:",
	  (double)stats1->sigattempts / timeinterval / (double)numnodes,
	  (double)stats2->sigattempts / timeinterval / (double)numnodes);
  printf ("  %-25s\t%.4f\t\t%.4f\n", "Sig. attempts per admitted:",
	  stats1->sigattpersucccall.currentrate, stats2->sigattpersucccall.currentrate);
  printf ("  %-25s\t%.4f\t\t%.4f\n", "Sig. attempts per blocked:",
	  stats1->sigattperfailcall.currentrate, stats2->sigattperfailcall.currentrate);  
  printf ("  %-25s\t", "Route pruning policy:");
  printPrunePolicy(net1);  
  printf ("\t");  
  printPrunePolicy(net2);
  printf ("\n");
  printf ("  %-25s\t", "Distance function:");
  printDistFunction(net1);  
  printf ("\t\t");    
  printDistFunction(net2);
  printf ("\n");
  printf ("  %-25s\t", "Link-state accuracy:");
  printStaleness();
  printf ("\n");
  printf ("%-20s\n", "Route stats:");    
  printf ("  %-25s\t%.3f\t\t%.3f\n", "Hopcount to minhop ratio:",
	  (float) stats1->sumhops / (float) stats1->summinhops,
	  (float) stats2->sumhops / (float) stats2->summinhops);
  printf ("  %-25s\t%.3f\t\t%.3f\n", "Avg. hopcount (success):",
	  (float) stats1->sumhops / (float) stats1->terminatedcalls,
	  (float) stats2->sumhops / (float) stats2->terminatedcalls);
  printf ("  %-25s\t%.3f\t\t%.3f\n", "Avg. hopcount (total):",
	  (float) stats1->requestedhc / (float)totalcalls,
	  (float) stats2->requestedhc / (float)totalcalls);  
  printf ("  %-25s\t%.3f\n", "Calc. avg. hopcount:", avghopcount);
  printf ("  %-25s\t%.3f\t\t%.3f\n", "Avg. minhopcount:",
	  (float) stats1->summinhops / (float) totalcalls,
	  (float) stats2->summinhops / (float) totalcalls);  
  printf ("  %-25s\t%d\t\t%d\n", "Num. long routes:",
	  stats1->longroutes, stats2->longroutes);
  printf ("  %-25s\t%.3f\t\t%.3f\n", "Avg. hopcount diff:",
	  (stats1->longroutes == 0) ? 0 : ((float) stats1->sumhopdiff / (float) stats1->longroutes),
	  (stats2->longroutes == 0) ? 0 : ((float) stats2->sumhopdiff / (float) stats2->longroutes));
  printf ("  %-25s\t%d\t\t%d\n", "Max hopcount diff:", stats1->maxhopdiff, stats2->maxhopdiff);
  printDualPerHopcountStats(net1, net2);
  printf ("%-20s\n", "Link update stats:");
  printf ("  %-25s\t%.3f\t\t%.3f\n", "Link update rate:",
	  (float) (stats1->rlinkupdates + stats1->tlinkupdates) / timeinterval,
	  (float) (stats2->rlinkupdates + stats2->tlinkupdates) / timeinterval);
  printf ("  %-25s\t%.3f\t\t%.3f\n", "Per-link update rate:",
	  (double) (stats1->rlinkupdates + stats1->tlinkupdates) / (timeinterval * (double)numlinks),
	  (double) (stats2->rlinkupdates + stats2->tlinkupdates) / (timeinterval * (double)numlinks));  
  printf ("  %-25s\t%.4f\t\t%.4f\n", "Avg. update interval:",
	  stats1->updinterval.currentrate, stats2->updinterval.currentrate);
  printf ("  %.1f%% %-25s\t[%.4f, %.4f]\n", confidence, "Upd. interval spread:",
	  stats1->updinterval.conf.lb, stats1->updinterval.conf.ub);
  printf ("  %-25s\t%d\t\t%d\n", "Total link updates:",
	  stats1->rlinkupdates + stats1->tlinkupdates,
	  stats2->rlinkupdates + stats2->tlinkupdates);
  printf ("  %-25s\t%d\t\t%d\n", "Refresh link updates:",
	  stats1->rlinkupdates, stats2->rlinkupdates);
  printf ("  %-25s\t%d\t\t%d\n", "Triggered link updates:",
	  stats1->tlinkupdates, stats2->tlinkupdates);
  printf ("  %-25s\t%d\t\t%d\n", "Deferred link updates:",
	  stats1->deferredupdates, stats2->deferredupdates);
  printf ("  %-25s\t%d\t\t%d\n", "Extraneous updates:",
	  stats1->extraupdates, stats2->extraupdates);
  printf ("  %-25s\t%.4f\t\t%.4f\n", "Time since update (blocked):",
	  stats1->timesincelinkupdfail.currentrate, stats2->timesincelinkupdfail.currentrate);
  printf ("  %-25s\t%.4f\t%.4f\n", "Time since update (admitted):",
	  stats1->timesincelinkupdsucc.currentrate, stats2->timesincelinkupdsucc.currentrate);  
  printf ("%-20s\n", "Traffic stats:");
  printf ("  %-25s\t%.4f\t%.4f\n", "Avg. residual bandwidth:",
	  stats1->residual.currentrate, stats2->residual.currentrate);
  printf ("  %.1f%% %-25s\t[%.4f, %.4f]\n", confidence, "Residual bw spread:",
	  stats1->residual.conf.lb, stats1->residual.conf.ub);
  printf ("  %-25s\t%.3f\n", "Maximumum requested bw:",
	  stats2->reqbwstat.max);
  printf ("  %-25s\t%.3f\n", "Minimum requested bw:",
	  stats2->reqbwstat.min);
  printf ("  %-25s\t", "Arrival rate spec:");
  printArrivalSpec ();
  printf ("\n");    
  printf ("  %-25s\t%.3f\n", "Max. interarrival time:",
	  stats2->arrivalratestat.max);
  printf ("  %-25s\t%.3f\n", "Min. interarrival time:",
	  stats2->arrivalratestat.min);
  printf ("  %-25s\t%.3f\n", "Avg. interarrival time:",
	  stats2->arrivalratestat.currentrate);  
  printf ("  %-25s\t%.3f\n", "Avg. arrival rate:",
	  1.0/stats2->arrivalratestat.currentrate);  
  printf ("  %-25s\t", "Holding time spec:");
  printHoldSpec ();
  printf ("\n");
  printf ("  %-25s\t%.3f\n", "Avg. req. holding time:",
	  stats2->holdtimestat.currentrate);  
  printf ("  %-25s\t%.3f\n", "Max req. holding time:",
	  stats2->holdtimestat.max);
  printf ("  %-25s\t%.3f\n", "Min req. holding time:",
	  stats2->holdtimestat.min);
  printf ("  %-25s\t%.3f (%.3f)\t%.3f (%.3f)\n", "Max succ. holding time:",
	  stats1->maxsuccduration, stats1->maxdurationtime,
	  stats2->maxsuccduration, stats2->maxdurationtime);
  printf ("  %-25s\t%.3f\n", "Holding time variance:",
	  stats2->holdtimestat.currentvar);
  printf ("%-20s\n", "Link stats:");
  printDualLinkUtilStats(net1, net2);
  printf ("  %-25s\t%.4f\n", "Time underutilized:", underutilstat.proportion);
  printf ("  %-25s\t%.4f\n", "Time-flows underutilized:", underutilflowstat.proportion);  
  printf ("  %-25s\t%.4f\n", "Avg. underutilization:", wunderutil / underutilstat.numersum);
  printf ("  %-25s\t%.4f\n", "Time overutilized:", overutilstat.proportion);
  printf ("  %-25s\t%.4f\n", "Time-flows overutilized:", overutilflowstat.proportion);  
  printf ("  %-25s\t%.4f\n", "Avg. overutilization:", woverutil / overutilstat.numersum);  
  printf ("  %-25s\t%.4f\n", "Time in violation:", violutilstat.proportion);
  printf ("  %-25s\t%.4f\n", "Time-flows in violation:", violutilflowstat.proportion);
  printf ("  %-25s\t%.4f\n", "Avg. violation:", wviolutil / violutilstat.numersum);  
  printf ("  %-25s\t%.3f\n", "Total link util stat time:",
	  (underutilstat.numersum + overutilstat.numersum + violutilstat.numersum));
  printDualCallsPerLinkStats (net1, net2);
  printf ("  %-25s\t%d\n", "Total num links:", numlinks);
#if RS_TRACE_LEVEL >= TRACE_ALL
  printCallTable (totalcalls);
#endif
}
/********************************************************************************/
#if RS_TRACE_LEVEL >= TRACE_ALL
static void
printCallTable (int totalcalls)
{
  int	i, j, numnodes;
  int	sumcalls = 0;

  numnodes = getNumNodes (rsnetwork);
  for (i = 0; i < numnodes; i++) {
    for (j = 0; j < numnodes; j++) {
      printf ("%d->%d: %.3f\n", i, j, ((double) calltable[i][j] / (double) totalcalls));
      sumcalls += calltable[i][j];
    }
  }
  if (sumcalls != totalcalls)
    printf ("Call count mismatch: sumcalls %d totalcalls %d\n", sumcalls, totalcalls);
}
#endif

