/* $Id: cdf.c,v 1.1 1999/06/11 17:12:16 ashaikh Exp $ */

#include "usercdf.h"
#include <stdio.h>
#include <stdlib.h>

/*
 *  Description:
 *
 *  This program can split a cdf at a given value, compute the average value of a cdf or
 *  print the cdf in an x-y format suitable for graphing.  Takes a routesim-formatted cdf
 *  on stdin.
 */
 
#define CDFFILE "cdf.dat"
#define CDFOUT "cdf.out"
#define LOWERCDF "lowercdf.dat"
#define UPPERCDF "uppercdf.dat"


static void printSplitCDF (cdf_t *, double);
static void printCDFAvg (cdf_t *cdf);
static void testCDF (cdf_t *);
/********************************************************************************/
int
main (int argc, char **argv)

{
  int c;
  extern char	*optarg;
  extern int	optind;
  int		errflg = 0;
  cdf_t		*cdf;
  double	split;



  while ((c = getopt(argc, argv, "s:ga")) != EOF)
    switch (c) {
    case 's':
      split = strtod (optarg, (char **) NULL);
      if (readCDFStdin (&cdf)) {
	fprintf (stderr, "readCDFFile failed\n");
	exit (1);
      }      
      printSplitCDF (cdf, split);
      break;
    case 'g':
      if (readCDFStdin (&cdf)) {
	fprintf (stderr, "readCDFFile failed\n");
	exit (1);
      }      
      printCDFGraph (cdf);
      break;
    case 'a':
      if (readCDFStdin (&cdf)) {
	fprintf (stderr, "readCDFFile failed\n");
	exit (1);
      }      
      printCDFAvg(cdf);
      break;
   case '?':
     errflg++;
   }
 if (errflg || (optind == 1)) {
   fprintf (stderr, "usage: cdf [-s <split>] [-g ] [-a]\ninput file: %s\n", CDFFILE);
   exit (2);
 }      


  return (0);
}

/********************************************************************************/
static void
printSplitCDF (cdf_t *cdf, double split)
{
  cdf_t		*upper, *lower;

  if (splitCDF (cdf, split, &lower, &upper)) {
    fprintf (stderr, "splitCDF failed.  Exiting ...\n");
    exit (1);
  }

  dumpCDFFile (lower, LOWERCDF);
  dumpCDFFile (upper, UPPERCDF);

  printf ("<= %f: ", split);
  printCDFAvg (lower);
  printf ("> %f: ", split);
  printCDFAvg (upper);
  
}
/********************************************************************************/
static void
printCDFAvg (cdf_t *cdf)
{
  printf ("Avg: %f\n", getCDFAvg (cdf));
  printf ("Percentiles:\n");
  printf ("\t0%%:   %f\n", getCDFVal (cdf, 0.0));  
  printf ("\t25%%:  %f\n", getCDFVal (cdf, .25));
  printf ("\t50%%:  %f\n", getCDFVal (cdf, .5));
  printf ("\t75%%:  %f\n", getCDFVal (cdf, .75));
  printf ("\t95%%:  %f\n", getCDFVal (cdf, .95));
  printf ("\t99%%:  %f\n", getCDFVal (cdf, .99));
  printf ("\t100%%: %f\n", getCDFVal (cdf, 1.0));
    
}

/********************************************************************************/
static void
testCDF (cdf_t *cdf)
{
  char	resp[25];
  double    u, x;
  int	    result;
  
  while (1) {
    printf ("Rand val (0 <= x <= 1): ");
    if (fgets (resp, 25, stdin) == (char *) 0) {
      printf ("reached EOF\n");
      break;
    }
    result = sscanf (resp, "%lf\n", &u);
    if (result != 1) {
      printf ("sscanf error\n");
      break;
    }
    if (u < 0 || u > 1) {
      printf ("Invalid\n");
      break;
    }
    x = getCDFVal (cdf, u);
    printf ("u = %.6f, x = %.6f\n", u, x);
  }
}
/********************************************************************************/	    
