/* $Id: weibull.c,v 1.1 1999/06/11 20:28:35 ashaikh Exp $ */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

/*
 * Description:
 *
 * This program computes the mean of a Weibull distribution given a shape and scale
 * parameter.  Also, given the desired mean and a shape value, it can compute the
 * necessary scale value.  Finally the 'g' option computes Weibull random variables
 * to test that a given scale and shape give the expected mean.
 */


#define RUNS	10000

static double getGamma (double);
static int runWeibull (double, double);
static int calcScale (double, double);
static int calcWeibull (double, double);

/********************************************************************************/
int
main (int argc, char **argv)
{
 int c;
 extern char *optarg;
 extern int optind;
 double scale, shape, mean;
 int errflg = 0;


 while ((c = getopt(argc, argv, "c:g:s:")) != EOF)
   switch (c) {
   case 'c':
     scale = strtod (optarg, (char **) NULL);
     shape = strtod (argv[optind++], (char **) NULL);
     calcWeibull(scale, shape);
     break;
   case 'g':
     scale = strtod (optarg, (char **) NULL);
     shape = strtod (argv[optind++], (char **) NULL);
     runWeibull(scale, shape);
   case 's':
     shape = strtod (optarg, (char **) NULL);
     mean = strtod (argv[optind++], (char **) NULL);
     calcScale (shape, mean);
     break;
   case '?':
     errflg++;
   }
 if (errflg || (optind == 1)) {
   fprintf (stderr, "usage: %s [-r <scale> <shape>] [-s <shape> <mean>]\n", argv[0]);
   exit (2);
 }

 return (0);
}

/********************************************************************************/
static int
calcWeibull (double scale, double shape)
{
  double    avg;

  if ((shape <= 0) || (scale <= 0)) {
    fprintf (stderr, "Shape and scale must be > 0\n");
    return (-1);
  }

  avg = (scale / shape) * getGamma ((1.0 / shape));
  printf ("Scale: %f Shape: %f Avg interarrival: %f\n", scale, shape, avg);

  return (0);
}

/********************************************************************************/
static int
calcScale (double shape, double mean)
{
  double scale;
  
  if (shape <= 0) {
    fprintf (stderr, "Shape and scale must be > 0\n");
    return (-1);
  }

  scale = shape * mean / getGamma ((1.0 / shape));
  printf ("Avg interarrival: %f Shape: %f Scale: %f\n", mean, shape, scale);

  return (0);
}

/********************************************************************************/
static int
runWeibull (double shape, double scale)
{
  int	i;
  double    inter, u, ln, exponent;

  double    sum;


  if ((shape <= 0) || (scale <= 0)) {
    fprintf (stderr, "Shape and scale must be > 0\n");
    return (-1);
  }

  srand48(1013553468);
  sum = 0;
  for (i = 0; i < RUNS; i++) {
    u = drand48();
    ln = -(log (u));
    exponent = (1.0 / shape);
    inter = scale * pow (ln, exponent);
    printf ("%g %g\n", u, inter);
    sum += inter;
  }
  fprintf (stderr, "Measured average interarrival:  %f\n", sum/(double)RUNS);

  return (0);
}
    
/********************************************************************************/  
static double
getGamma (double x)
{
  double lg;
  
  lg = lgamma(x);
  return (signgam*exp(lg));
}
