.TH rsfiles 4 "April 1999"

.SH NAME

rsfiles \- Description of
.B routesim
simulator file formats

.SH SYNOPSIS

.B rstopology.dat
.br
.B rstopology.dat.out
.br
.B rstraffic.dat
.br
.B holdcdf.dat
.br
.B rsmonitor.dat
.br
.B rslinkmonitor.log
.br
.B routesim.log
.br
.B configuration file


.SH DESCRIPTION

.TP 5
.B rstopology.dat
.RS
The rstopology file describes a arbitrary network topology for use in
a
.B routesim
simulation.  Each line of the topology file specifies a
network link, and all links are unidirectional.  The format of a
topology file is as follows:

.I num nodes
.br
.I from to capacity propdelay adminweight maxcalls
.br
\|.
.br
\|.
.br
\|.

Items are separated by a space.  The first line in the file is an
integer indicating the number of nodes in the network.  Node ids are
in the range 0 to
.I num nodes
- 1. Each subsequent line defines a link and has 6 fields.
.I from
and
.I to
specify the nodes the link connects.  The third, fourth, and fifth
fields are floating point numbers.  Link capacity is specified
in the third field.  Propagation delay is specified by the fourth
field.  An arbitrary administrative weight is in the fifth field.
The final field is the maximum number flows that can be supported on
the link (integer).  Each field, except
.I from
and
.I to
may be set to -1.0 (or -1 for
.I maxcalls
) to get assigned the default value.

.RE
.TP 5
.B rstopology.dat.out
.RS

This file is dumped when the -dumptop or dump-topology options are
specified to
.I routesim.
It follows the format described for
.B rstopology.dat
above.


.RE
.TP 5
.B rstraffic.dat
.RS

The rstraffic file specifies a
.B routesim
traffic matrix.  The traffic matrix specifies the arrival rate between
each source-destination pair.  It also allows different values for
different times-of-day.  The arrival rate is expressed in terms of a
scale value and shape value that serve as parameters to the flow
arrival distribution.  The possible distributions are Poisson and
Weibull, and is specified in a
.B routesim
configuration file or on the command line.  In the case of
Poisson, only the scale value is significant, and it represents the
mean value.  For the Weibull distribution, both the scale and shape
values are significant.  Together they determine the mean of the
distribution.  The file format is:

.I from to mscale mshape ascale ashape
.I escale eshape
.br
\|.
.br
\|.
.br
\|.

Each line describes a traffic rate from the node
.I from
to the node 
.I to.
The
.I mscale
,
.I ascale
, and
.I escale
are floating point values that specify the scale parameters for
morning, afternoon, and evening, respectively.
.I mshape
,
.I ashape
, and
.I eshape
are the corresponding shape parameters.

.I Note: scales are the arrival
.I rates -- NOT interarrival times.

.RE
.TP 5
.B holdcdf.dat
.RS

The holdcdf file specifies an inverse cumulative distribution
function for the flow durations.  The file format is:

.I num entries
.br
.I u x_first
.br
\|.
.br
\|.
.br
\|.
.br
1
.I x_last

The first line specifies the number of entries that follows.  Each
subsequent line is a single (
.I u
,
.I x
) entry, in ascending
.I u
order.  The first entry should NOT be for
.I u
= 0.  But the last entry must be for
.I u
= 1.  See examples below for further explanation.

.RE
.TP 5
.B rsmonitor.dat
.RS

The rsmonitor file specifies the network links that should be
monitored during simulaton.  It's format is simply:

.I from to
.br
\|.
.br
\|.
.br
\|.

Each line specifies the node ids for the link endpoints.  Typically
only a few links (maybe just one) are specified since the monitoring
information copious.

.RE
.TP 5
.B rslinkmonitor.log
.RS

The rslinkmonitor file is the log of events on the links specified in
.B rsmonitor.dat

.RE
.TP 5
.B routesim.log
.RS

This file logs the progress of the
.I routesim
simulation.  It lists the phase (warmup or actual sim), current
simulation time, number of flow requests, and confidence interval for
the stopping metric.

.RE
.TP 5
.B configuration file
.RS

The configuration file contains
.I routesim
options, one per line, in long format.  It may also contain comment
lines with /* ... */ delimiters.  Note that there must be a space
between the '*' character and the comment text.

.SH EXAMPLES

.RE
.TP 5
.B rstopology.dat
.RS

16
.br
0 1 1.000000 0.000000 0.000000 2147483647
.br
0 3 1.000000 0.000000 0.000000 2147483647
.br
0 4 1.000000 0.000000 0.000000 2147483647
.br
0 12 1.000000 0.000000 0.000000 2147483647
.br
1 0 1.000000 0.000000 0.000000 2147483647
.br
1 2 1.000000 0.000000 0.000000 2147483647
.br
1 5 1.000000 0.000000 0.000000 2147483647
.br
\|.
.br
\|.
.br
\|.

.RE
.TP 5
.B rstraffic.dat
.RS

0 1 0.150000 1.000000 0.150000 1.000000 0.150000 1.000000
.br
0 2 0.113208 1.000000 0.113208 1.000000 0.113208 1.000000
.br
0 3 0.150000 1.000000 0.150000 1.000000 0.150000 1.000000
.br
0 4 0.113208 1.000000 0.113208 1.000000 0.113208 1.000000
.br
0 5 0.113208 1.000000 0.113208 1.000000 0.113208 1.000000
.br
\|.
.br
\|.
.br
\|.
2 0 0.113208 1.000000 0.113208 1.000000 0.113208 1.000000
.br
2 1 0.150000 1.000000 0.150000 1.000000 0.150000 1.000000
.br
2 3 0.150000 1.000000 0.150000 1.000000 0.150000 1.000000
.br
2 4 0.113208 1.000000 0.113208 1.000000 0.113208 1.000000
.br
2 5 0.113208 1.000000 0.113208 1.000000 0.113208 1.000000
.br
2 6 0.113208 1.000000 0.113208 1.000000 0.113208 1.000000
.br
2 7 0.113208 1.000000 0.113208 1.000000 0.113208 1.000000
.br
\|.
.br
\|.
.br
\|.

.RE
.TP 5
.B holdcdf.dat
.RS

Consider the following CDF:

F(x) =
.RS 10
0.0	0 <= x < 64
.br
0.7	64 <= x < 128
.br
0.8	128 <= x < 256
.br
0.9	256 <= x < 512
.br
1.0	512 <= x

.RE
The corresponding inverse CDF is:
 -1
.br
F  (x) =
.RS 10
64	0 < u <= 0.7
.br
128	0.7 < u <= 0.8
.br
256	0.8 < u <= 0.9
.br
512	0.9 < u <= 1.0

.RE
and the corresponding holdcdf.dat file is:

4
.br
0.7 64
.br
0.8 128
.br
0.9 256
.br
1.0 512

.RE
.TP 5
.B routesim.log
.RS

A typical log file looks like:

Warming up ...
.br
847500 (0.0022 [0.0021, 0.0024]) (overutil) Time: 725.72
.br
Warmup complete: 848011 calls
.br
2655000 (0.0566 [0.0563, 0.0568]) (overutil) Time: 2998.96
.br
Simulation complete.  Dumping stats ...

.RE
.TP 5
.B configuration file
.RS

Sample configuration file:

topology-type file
.br
network-model SINGLE
.br
arrival-spec ARR_UNIFORM
.br
arrival-dist POISSON
.br
arrival-scale .94118
.br
arrival-shape 1.00
.br
traffic-multiplier 1.0
.br
time-of-day AFTERNOON
.br
holding-time-spec USERDEF
.br
holding-time-shape 2.5
.br
holding-time-scale 1.00
.br
mean-bandwidth 0.030
.br
bandwidth-spread 1.99000
.br
update-trigger 100000.0000
.br
refresh-interval 120.000000
.br
refresh-skew 0.02
.br
min-update-interval 0.000000
.br
min-requests 800000
.br
min-warmup 500000
.br
min-sim-time 3000.0
.br
warmup-proportion 0.25
.br
blocking-policy NON_BLOCKING
.br
routing-policy ON_DEMAND
.br
routing-algorithm WIDE_SHORT
.br
distance-function HOPCOUNT
.br
hopcount-threshold 1
.br
wide-short-maxhops 10
.br
alternate-routing NO_ALT
.br
multi-route-policy UNIQ
.br
uniq-route-policy RANDOM
.br
max-sig-attempts 1
.br
prune-policy NO_PRUNE
.br
link-state-accuracy STALE
.br
random-seed 1013553468
.br
confidence-level 99
.br
sim-tolerance .05
.br
warmup-tolerance .075

.SH "SEE ALSO"

.BR routesim(l)

.SH AUTHOR

Anees Shaikh (University of Michigan, AT&T Labs--Research, 1997-99)
