/* $Id: opt-parse.c,v 1.6 1998/02/01 05:55:05 ashaikh Exp $ */

#define _OPT_PARSE_C_
#include <stdio.h>
#include <sys/types.h>		/* for NBBY, number of bits/byte */
#include <stdlib.h>		/* for calloc(),atoi(),etc. */
#include <string.h>		/* for strtok() */
#include <sys/time.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <strings.h>
#include <stdarg.h>

#include "general.h"
#include "opt-parse.h"
/* printout options */
#define OPTION TRUE
#define ALWAYS TRUE

op_struct global_options[MAX_OPTIONS];
int options_used = 0;
char CONF_FILE_NAME[255];
void c_printf (boolean c, ...);
void panic(char *s);

void parse_conf_file (char *filename){
  FILE *cf;
  int i;
  char **options;
  int  num_options = 0;
  /* allows recursion !*/

  strcpy (CONF_FILE_NAME, filename);
  
  if ((cf = fopen (filename, "r"))==NULL){
    c_printf (ALWAYS, "Could not open file %s\n", filename);
    exit(-1);
  }

  /* allocate space for the options */
  options = (char **)getmem ((sizeof(char *)*MAX_OPTIONS));
  for (i=0; i < MAX_OPTIONS; i++) {
    options[i] = (char *) getmem(255);
  }

  /* first option is skipped */
  strcpy (options[num_options++], "csim beta v0");
  for (;fscanf (cf, "%s", &options[num_options][0])!=EOF;){
    //     printf ("option %s\n", &options[num_options][0]);
    
    num_options++;
    if (num_options >= MAX_OPTIONS){
      c_printf (ALWAYS, "option parser memory exhausted on file %s\n",
		filename);
      exit(1);
    }
  }
  
  /* now parse the options */
  parse_command_line (num_options, options);

  /* and free the options */
  for (i=0; i < MAX_OPTIONS; i++) {
    freemem(options[i]);
  }
  freemem (options);
}


void usage (void){
  int i;
  
  for (i=0; i < options_used; i++){
    printf ("%s \t%s\n  -- %s\n\n",
	    global_options[i].l_name,
	    global_options[i].s_name,
	    global_options[i].des);
  }
}

void shortusage (void){
  int i;

  printf("\n");
  
  for (i=0; i < options_used; i++){
    printf ("\t%-14s\t%s\n",
	    global_options[i].s_name,	    
	    global_options[i].des);
  }
}


void insert_option(char *s0, char *s1,
		   option_type t, void *var,
		   char *d,
		   void * validp,
		   void * offset){
  if ((options_used)>=MAX_OPTIONS)
    panic("option table memory exhausted\n");

  global_options[options_used].index = options_used;
  strcpy(global_options[options_used].s_name, s0);
  strcpy(global_options[options_used].l_name, s1);
  strncpy (global_options[options_used].des, d, 254);
  
  global_options[options_used].type = t;
  global_options[options_used].var = var;

  global_options[options_used].validp = validp;
  global_options[options_used].generic_offset = offset;

  options_used++;
	
  // typedef enum {
  //   boolean_op, float_op, int_op, string_op, generic_op
  // } option_type;
}

int submatch (char *a, char *b){
  int i = 0;
  
  for (;(a[i]&&b[i]);){
    if (a[i]==b[i])
      i++;
    else
      return i;
  }
  return i;
}


int strmatch (char *target, char *s, char *l){
  /* get best unique match in table */
  /* if s or l return 1 else return 0 */
  int i;
  int j = -1;
  int k ;
  int targets_match = -1;

  k = submatch(s, target);
  targets_match = submatch(l, target);
  if (k > targets_match)
    targets_match = k;

  if (targets_match < strlen(target)){
    //      printf ("target %s, option %s, strlen %d, targets_match %d\n",
    //  	    target, l,
    //  	    strlen(target), targets_match);
    return 0;
  }
  /* has to match the whole target */
  
  /* we know how large the target match is */
  //  printf ("target-match %d\n", targets_match);
  
  for (i=0; i < options_used; i++){
    k = submatch(global_options[i].s_name, target);
    if (k > targets_match)
      return 0;
    else
      if ((k == targets_match)&&(strcmp(s, global_options[i].s_name))){
	/* ambiguity */
	c_printf (OPTION, "ambiguous option %s %s %s\n",
		  target, s, global_options[i].s_name);
	return 0;
      }
    k = submatch(global_options[i].l_name, target);
    if (k > targets_match)
      return 0;
    else
      if ((k == targets_match)&&(strcmp(l, global_options[i].l_name))){
	/* ambiguity */
	c_printf (OPTION, "ambiguous option %s %s %s\n",
		  target, l, global_options[i].l_name);
	return 0;
      }
  }
  // c_printf ("best match %s\n", global_options[j].s_name);
  return 1;
}


boolean toggle (boolean var){
  if (var == TRUE)
    return FALSE;
  return TRUE;
}

void boolean_assign (boolean *var, boolean *val){
  *var = toggle(*var);
}

void float_assign (float *var, char *val){
  *var = atof (val);
}

void double_assign (double *var, char *val){
  *var = strtod (val, (char **) NULL);
}

void long_assign (long *var, char *val){
  *var = strtol (val, (char **) NULL, 10);
}

void int_assign (int *var, char *val){
  *var = atoi (val);
}

void string_assign (char *var, char *val){
  strcpy (var, val);
}

void name_match (int num_options, char table[][GEN_STR_TABLE_WIDTH],
		 char *target, void *var){
  int i;
  int found = FALSE;

  for (i = 0; (i < num_options) && (table[i]); i++){		
    if (!strncasecmp(target,
		     table[i], strlen(target))){
      bcopy (&i, var, sizeof(i));
      found = TRUE;
    }
  }
  if (!found)
    c_printf (ALWAYS, "error parsing name match %s\n", target);
}



boolean match (int *argc, int *index, char **argv){
  /* look at argv[index]
   * if strmatch
   *     then assign option
   *    return true
   * return false
   */
#define check_args0() if (*argc < 0) panic ("parameter parser: input exhausted")
  int i;
  for (i=0; i < options_used; i++){
    if (strmatch(argv[*index], global_options[i].s_name,
		 global_options[i].l_name)){
      if (global_options[i].type == generic_op){
	*argc-=2;
	*index+=1;
	check_args0();
	name_match(MAX_GEN_TABLE_SIZE,
		   (char (*)[])global_options[i].generic_offset,
		   argv[(*index)++],
		   (void *)global_options[i].var);
      }
			
      if (global_options[i].type == boolean_op){
	*argc-=1;
	*index+=1;
	boolean_assign ((boolean *)global_options[i].var,
			NULL);
				
      }
      if (global_options[i].type == float_op){
	*argc-=2;
	*index+=1;
	check_args0();
	float_assign ((float *)global_options[i].var,
		      argv[(*index)++]);
				
      }
      if (global_options[i].type == double_op){
	*argc-=2;
	*index+=1;
	check_args0();
	double_assign ((double *)global_options[i].var,
		      argv[(*index)++]);
				
      }
      if (global_options[i].type == long_op){
	*argc-=2;
	*index+=1;
	check_args0();
	long_assign ((long *)global_options[i].var,
		      argv[(*index)++]);
				
      }      
      if (global_options[i].type == funcall){
	*argc-=2;
	*index+=1;
	check_args0();
	{
	  void (*f)(void *);
	  f = (void (*)(void *))global_options[i].var;
	  f(argv[(*index)++]);
	}
      }
      if (global_options[i].type == string_op){
	*argc-=2;
	*index+=1;
	check_args0();
	string_assign ((char *)global_options[i].var,
		       argv[(*index)++]);
      }
      if (global_options[i].type == int_op){
	*argc-=2;
	*index+=1;
	check_args0();
	int_assign ((int *)global_options[i].var,
		    argv[(*index)++]);
      }
      return TRUE;
    }
  }
  return FALSE;
}



int NO = 0;
void parse_command_line (int argc, char**argv){

#define CONF_FILE0 "-conf"
#define CONF_FILE1 "configuration-file"

#define USAGE0 "-usage"
#define USAGE1 "-usage"

#define BEGIN_COMMENT0 "/*"
#define BEGIN_COMMENT1 "%"
#define END_COMMENT0 "*/"
#define END_COMMENT1 "%"
  
  int index;

  static conf_file_in = 0;

  if (!conf_file_in){
    insert_option ("-conf",   "configuration-file",
		   funcall,
		   (void *)parse_conf_file,
		   "Name of Configuration File",
		   (void *) &NO,
		   0x0);
  }
  
  if (!(--argc)){
    shortusage();
    exit (0);
    return;
  }

  //#define strmatch(target, m1, m2) ((!strncmp(target, m1, strlen(m1))) ||\
  //                                 (!strncmp(target, m2, strlen(m2))))

#define	parse_name(a,b,c,d) \
  argc-=2;\
  index++;\
  check_args();\
  name_match (a,b,c,d);
	
#define check_args() if (argc < 0) panic ("parameter parser: input exhausted")
  
  for (index = 1; argc > 0;){
    if (strmatch(argv[index],  USAGE0, USAGE1)){
      argc--;
      index++;
      check_args();
      usage();
      exit (0);
      continue;
    }
    /* eat all tokens after begin_comment till end_comment */
    if (strmatch(argv[index], BEGIN_COMMENT0, BEGIN_COMMENT1)){
      argc--;
      index++;
      check_args();
      for (;(argc > 0)&&(!strmatch(argv[index], END_COMMENT0, END_COMMENT1));){
/* 	 printf ("eating %s\n", argv[index]); */
	argc--;
	index++;
	check_args();
      }
      /*
       * either
       *    0. argc ran out
       * or 1. we are looking at the end comment
       */
      if (argc > 0){
	/* eat the end of comment marker */
	argc--;
	index++;
	check_args();
      }
      else {
	c_printf (ALWAYS,
		  "Unterminated Comment? last token %s\n", argv[index-1]);
	exit(-4);
      }
      continue;
    }
    if (match(&argc, &index, argv)){
      /* magic! */
      continue;
    }
    {
      c_printf (ALWAYS, "parse error, token %s\n", argv[index]);
      exit (-3);
    }
  }
}

void option_print(int i, char *o, char *v){
  c_printf((boolean)
	   ((OPTION) &&
	   (*(boolean *)(global_options[i].validp))),
	    "%s %s\n", o, v);
}

void dump_state (){
  {
    int i;
    c_printf (OPTION, "\n---\n");
    for (i=0; i < options_used; i++){
      if (global_options[i].type == boolean_op){
	if ((*(int *)global_options[i].var)==TRUE){
	  option_print (i, global_options[i].l_name, "");
	}
	/* add the else # clause soon */
      }
      if (global_options[i].type == float_op){
	char v[255];
	sprintf (v, "%f", *(float *)global_options[i].var);
	option_print (i, global_options[i].l_name, v);
      }
      if (global_options[i].type == double_op){
	char v[255];
	sprintf (v, "%f", *(double *)global_options[i].var);
	option_print (i, global_options[i].l_name, v);
      }      
      if (global_options[i].type == string_op){
	option_print (i, global_options[i].l_name,
		      (char *)global_options[i].var);
      }
      if (global_options[i].type == generic_op){
	if ((char *)global_options[i].var){
	  option_print (i, global_options[i].l_name,
			(char *)((char (*)[GEN_STR_TABLE_WIDTH])
				 global_options[i].generic_offset)
			[*(int *)global_options[i].var]);
	}
      }
      if (global_options[i].type == int_op){
	char v[255];
	sprintf (v, "%d", *(int *)global_options[i].var);
	option_print (i, global_options[i].l_name, v);
      }
      if (global_options[i].type == long_op){
	char v[255];
	sprintf (v, "%ld", *(long *)global_options[i].var);
	option_print (i, global_options[i].l_name, v);
      }      
    }
  }
  c_printf (OPTION, "---\n");
  return;
}

void c_printf (boolean c, ...){
  /*
   * conditional printer -- print iff
   * c is true -- thanks to bodhi at
   * chaos for the details
   */
  va_list ap;
  char *f;
  if (!c)
    return;
  va_start(ap, c);
  f = va_arg (ap, char *);
  vprintf (f, ap);
  fflush(stdout);
  va_end (ap);
}


void panic (char *s)
{
  c_printf (ALWAYS, "panic in <%s>\n", s);
  exit(-1);
}

