/* $Id: rslinklog.c,v 1.6 1999/01/08 05:12:46 ashaikh Exp $ */


#include "rslinklog.h"
#include "rsdefs.h"
#include "rslog.h"
#include "rstrace.h"
#include <stdio.h>


extern double minupdinterval;

static void writeCall (char, double, link_t *, call_t *, int);
static void writeLink (char, double, link_t *, int);
/********************************************************************************/

int
logLinkEvent (linkevent_t type, double time, link_t *link, call_t *call, int logid)
{
  
  switch (type) {
  case LINK_ARR:
    writeCall ('a', time, link, call, logid);
    break;
  case LINK_TERM:
    writeCall ('t', time, link, call, logid);
    break;
  case LINK_TRIG:
    writeLink ('u', time, link, logid);
    break;
  case LINK_PEND:
    writeLink ('p', time, link, logid);
    break;
  case LINK_SIGFAIL:
    writeLink ('f', time, link, logid);
    break;
  case LINK_UTIL:
    writeLink ('b', time, link, logid);
    break;
  case LINK_UTIL_DYN:
    writeLink ('d', time, link, logid);
    break;
  case LINK_UTIL_STAT:
    writeLink ('s', time, link, logid);
    break;    
  default:
    RS_PRINT_SOFT_ERROR ("logLinkEvent: Unknown link event type\n");
    return (-1);
  }
  
  return (0);
}

int
logLinkRefreshEvent (double time, double refreshtime, link_t *link, int logid)
{
  char	logbuf[100];

  /* format:
   *  refresh
   *  time
   *  src->dest
   *  advertised
   *  upd.interval
   */
  sprintf (logbuf, "r %.3f %d->%d %.2f %.2f\n",
	   time, 
	   link->from, link->to,
	   link->availbw.current,
	   refreshtime);
  rsWriteLog (logid, logbuf, 0);
  return (0);
}

/********************************************************************************/
static void
writeCall (char t, double time, link_t *link, call_t *call, int logid)
{
  char	logbuf[100];

  switch (t) {
  case 'a':
  case 't':
    /* output format:
     *  arrival
     *  time
     *  src->dest
     *  callid
     *  call bandw
     *  occupancy
     *  available bw
     *  last adverstised bw
     *  inaccuracy
     */
    sprintf (logbuf, "%c %.3f %d->%d %d %.2f %.2f %.2f %.2f %.2f\n", t,
	     time,
	     link->from, link->to,
	     call->callid, call->bandw,
	     link->capacity - link->availbw.current,
	     link->availbw.current,
	     link->availbw.last,
	     (link->availbw.current - link->availbw.last));
    rsWriteLog (logid, logbuf, 0);
    break;
  default:
    break;
  }

}

/********************************************************************************/
static void
writeLink (char t, double time, link_t *link, int logid)
{
  char	logbuf[100];

  switch (t) {
  case 'u':
    /* format:
     *	update
     *	time
     *	src->dest
     *	advertisement
     *	update interval
     */
    sprintf (logbuf, "%c %.3f %d->%d %.2f %.2f\n", t,
	     time,
	     link->from, link->to,
	     link->availbw.current,
	     time - link->lastupdtime);
    rsWriteLog (logid, logbuf, 0);
    break;
  case 'f':
    /* format:
     *  sig failure
     *  time
     *  src->dest
     *  dummy val (0)
     */
    sprintf (logbuf, "%c %.3f %d->%d 0\n", t,
	     time,
	     link->from, link->to);
    rsWriteLog (logid, logbuf, 0);    
    break;    
  case 'p':
    /* format:
     *    update
     *    time
     *    src->dest
     *    advertisement
     *    update interval
     */          
    sprintf (logbuf, "%c %.3f %d->%d %.2f %.2f\n", t,
	     link->lastupdtime + minupdinterval,
	     link->from, link->to,
	     link->availbw.current,
	     minupdinterval);
    rsWriteLog (logid, logbuf, 0);
    break;
  case 'b':
  case 'd':
  case 's':
    /* output format:
     *  time
     *  src->dest
     *  avg utilization
     */
    sprintf (logbuf, "%c %.3f %d->%d %.3f\n", t,
	     time,
	     link->from, link->to,
	     (link->capacity - link->availbw.current)/link->capacity);
    rsWriteLog (logid, logbuf, 0);
    break;
  default:
    break;
  }
}
/********************************************************************************/
