#include "usercdf.h"
#include <stdio.h>
#include <stdlib.h>

/*  Description:
 *
 *  For hybrid routing algorithm
 *
 *  Given a cdf in the routesim format on stdin, and a flow trigger,
 *  this program computes the proportion of flows and traffic, as well
 *  as their avg. duration.
 *
 */

double	flowtrig;

static void printCapacitySplit (cdf_t *);
/********************************************************************************/
int
main (int argc, char **argv)
{
  int c;
  int		errflg = 0;
  extern char	*optarg;
  extern int	optind;
  cdf_t		*cdf;

  while ((c = getopt(argc, argv, "f:")) != EOF)
    switch (c) {
    case 'f':
      flowtrig = strtod (optarg, (char **) NULL);
      if (readCDFStdin (&cdf)) {
	fprintf (stderr, "readCDFFile failed\n");
	exit (1);
      }      
      printCapacitySplit (cdf);
      break;
   case '?':
     errflg++;
   }
  if (errflg || (optind == 1)) {
    fprintf (stderr, "usage: calcpartition -f <flow trigger>\n");
    exit (2);
  }            
}
/********************************************************************************/
static void
printCapacitySplit (cdf_t *cdf)
{
  cdf_t		*lower, *upper;
  double	u;
  int		pos;
  double	loavg, hiavg;
  double	locap, hicap, denom;
  
  for (pos = 0; pos < cdf->numvalues - 1; pos++) {
      if (cdf->values[pos].x > flowtrig)
	break;
  }
  u = cdf->values[pos-1].u;	
  splitCDF (cdf, u, &lower, &upper);
  loavg = getCDFAvg (lower);
  hiavg = getCDFAvg (upper);
  destroyCDF (lower);
  destroyCDF (upper);
  destroyCDF (cdf);
  denom = u * loavg + (1.0 - u) * hiavg;
  locap = (u * loavg + (1.0 - u) * flowtrig) / denom;
  hicap = 1.0 - locap;
  printf ("trigger: %f\nflow proportion low: %f\nlow avg.: %f\nhigh avg.:%f\ntraffic proportion low: %f\ntraffic proportion high: %f\n", flowtrig, u, loavg, hiavg, locap, hicap);
}

    
      
	
  
