/* $Id: genrandtop.c,v 1.1 1999/06/11 17:15:12 ashaikh Exp $ */

static const char rcsid[] = "$Id: genrandtop.c,v 1.1 1999/06/11 17:15:12 ashaikh Exp $";

/*  Description:
 *  
 *  An offline program to generate a random topology using either the Waxman model
 *  or a pure random model (i.e., flip a coin and add an edge).  Prints topology
 *  statistics and dumps the topology to a file suitable for direct use in routesim.
 */

#include "topology.h"
#include "rstrace.h"
#include <stdlib.h>

#define OUTPUT_TOPOLOGY_FILE    "rstopology.dat.out"

topology_t  *network;

int	    method = 0, asym = 0;
int	    size = 0;
int	    maxdiam = 0;
double	    alpha = 0, beta = 0, planesize = 0;
int	    dumptop = 0;

randgraph_t graphtype;
boolean	    asymflag;


static int getParms (int, char **);

/********************************************************************************/
int
main (int argc, char **argv)
{

  if (getParms(argc, argv)) {
    exit (1);
  }

  (method == 0) ? (graphtype = RG2) : (graphtype = PURE);
  (asym == 0) ? (asymflag = FALSE) : (asymflag = TRUE);

  srand48(1013553468);	/* total hack */

  if (createRandTopology (graphtype, size, alpha, beta, planesize, asymflag, maxdiam, &network)) {
    RS_PRINT_ERROR ("createRandTopology failed\n");
    exit (1);
  }

  printTopologyStats (network);

  if (dumptop) {
    dumpTopologyFile (OUTPUT_TOPOLOGY_FILE, network);
  }


  return (0);
}

/********************************************************************************/
static int
getParms (int argc, char **argv)
{
  int c;
  extern char *optarg;
  extern int optind;
  int errflg = 0;
  
   while ((c = getopt(argc, argv, "s:a:b:p:m:d:y:f")) != EOF)
   switch (c) {
   case 's':
     size = (int) strtol (optarg, (char **) NULL, 10);
     break;
   case 'a':
     alpha = strtod (optarg, (char **) NULL);
     break;
   case 'b':
     beta = strtod (optarg, (char **) NULL);
     break;     
   case 'p':
     planesize = strtod (optarg, (char **) NULL);
     break;
   case 'm':
     method = (int) strtol (optarg, (char **) NULL, 10);
     break;
   case 'd':
     maxdiam = (int) strtol (optarg, (char **) NULL, 10);
     break;
   case 'y':
     asym = (int) strtol (optarg, (char **) NULL, 10);
     break;
   case 'f':
     dumptop = 1;
     break;     
   case '?':
     errflg++;
   }
   if (errflg || (optind < 14)) {
     fprintf (stderr, "usage:\t%s\t-s <size>\n\t\t\t-a <alpha>\n\t\t\t-b <beta>\n\t\t\t-p <planesize>\n\t\t\t-m <method>\n\t\t\t-d <maxdiam>\n\t\t\t-y <asym>\n\t\t\t[-f]\n\tmethod: 0 - RG2, 1 - PURE\n\tasym: 0 - symmetric, 1 - asymmetric\n\ttopology file: %s\n", "genrandtop", OUTPUT_TOPOLOGY_FILE);
     return (-1);
   }

   return (0);
     
}
