#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <strings.h>

/*
 * Description:
 *
 * This program generates a skewed traffic matrix to simulate network hot spots.  It
 * dumps the traffic matrix file to stdout.
 *
 */


int	numnodes;
int	numhotnodes;
double	skew;
double	shape;
double	scale;
double	**matrix;


static void printUsage (void);
static void genEvenFile (void);
static void genHotFile (void);
/********************************************************************************/
int
main (int argc, char **argv)
{

  srand48 (1013553468);

  if (argc < 5 || argc > 6) {
    printUsage();
    exit(1);
  }

  if (argc == 5) {
    numnodes = (int) strtol (argv[1], (char **) NULL, 10);
    skew = strtod (argv[2], (char **) NULL);
    shape = strtod (argv[3], (char **) NULL);
    scale = strtod (argv[4], (char **) NULL);
    genEvenFile ();
  }

  if (argc == 6) {
    numnodes = (int) strtol (argv[1], (char **) NULL, 10);
    numhotnodes = (int) strtol (argv[2], (char **) NULL, 10);  
    skew = strtod (argv[3], (char **) NULL);
    shape = strtod (argv[4], (char **) NULL);
    scale = strtod (argv[5], (char **) NULL);
    genHotFile ();
  }

  return (0);
}
/********************************************************************************/
static void
printUsage (void)
{
   fprintf (stderr, "Usage: gentmatrix.pl <numnodes> [num hotnodes] <scale skew> <shape> <scale>\n");
}
/********************************************************************************/
static void
genEvenFile (void)
{
  int	    i, j;
  double    flip, newscale;
  double    sumscale;


  sumscale = 0.0;
  for (i = 0; i < numnodes; i++) {
    for (j = 0; j < numnodes; j++) {
      if (i == j) {
	continue;
      }
      printf ("%d %d ", i, j);
      flip = drand48();
      /* simple 3-way random choice */
      if (flip <= 0.33333) {
	newscale = scale;
      }
      else {
	if (flip > 0.33333 && flip < 0.66666) {
	  newscale = (1.0 - skew) * scale;
	}
	else {
	  newscale = (1.0 + skew) * scale;
	}
      }
      printf ("%f %f %f %f %f %f\n", newscale, shape, newscale, shape, newscale, shape);
      sumscale += newscale;
    }
  }
}
/********************************************************************************/
static void
genHotFile (void)
{
  int	i, j, id, *hotnodes;
  double    hotval, coldval, hotnodehotval, hotnodecoldval;
  double    nominal, newscale;

  nominal = scale * (numnodes - 1);
  coldval = (1.0 - skew) * nominal / (numnodes - 1 - numhotnodes);
  hotval = skew * nominal / (numhotnodes);
  hotnodecoldval = (1.0 - skew) * nominal / ((numnodes - 1) - (numhotnodes - 1));
  hotnodehotval = hotval + (hotval / (numhotnodes - 1));

  hotnodes = (int *) malloc (sizeof(int) * numnodes);
  bzero (hotnodes, sizeof(int)*numnodes);
  
  /* fill in the whole matrix with the nominal cold scale */
  matrix = (double **) malloc (sizeof(double *) * numnodes);
  for (i = 0; i < numnodes; i++) {
    matrix[i] = (double *) malloc (sizeof(double) * numnodes);
    for (j = 0; j < numnodes; j++) {
      if (i == j)
	continue;
      matrix[i][j] = coldval;
    }
  }

  /* generate ids of hot nodes */
  for (i = 0; i < numhotnodes; i++) {
    /* uniform between 0 and (numnodes)^2 - 1 */
    id = (int) floor(0 + (numnodes)*drand48());
    while (hotnodes[id]) {
      id = (int) floor(0 + (numnodes)*drand48());      
    }
    hotnodes[id] = 1;
  }

  /* update entries to hot nodes */
  for (i = 0; i < numnodes; i++) {
    for (j = 0; j < numnodes; j++) {
      if (i == j)
	continue;
      if (hotnodes[i]) {
	if (hotnodes[j]) {
	  matrix[i][j]= hotnodehotval;
	}
	else {
	  matrix[i][j]= hotnodecoldval;
	}
      }
      else {
	if (hotnodes[j]) {
	  matrix[i][j]= hotval;
	}
      }
    }
  }
  

  /* print the matrix */
  for (i = 0; i < numnodes; i++) {
    for (j = 0; j < numnodes; j++) {
      if (i == j)
	continue;
      printf ("%d %d ", i, j);
      newscale = matrix[i][j];
      printf ("%f %f %f %f %f %f\n", newscale, shape, newscale, shape, newscale, shape);
    }
  }
}
      
      
  
	    

