/* $Id: loadcalc.c,v 1.1 1999/06/11 20:02:57 ashaikh Exp $ */

static const char rcsid[] = "$Id: loadcalc.c,v 1.1 1999/06/11 20:02:57 ashaikh Exp $";

#include "topology.h"
#include <stdio.h>
#include <stdlib.h>

/*
 * Description:
 *
 * This program computes the offered load for a given topology and set of average traffic
 * parameters:  arrival rate, flow duration, and flow bandwidth.  The topology file is
 * read from stdin and should be in routesim format.
 *
 */

topology_t  *network;
double	    avgrate, avglen, avgbw;

static int getOptions (int, char **);
static void printOfferedLoad (void);

/********************************************************************************/
int
main (int argc, char **argv)
{

  if (getOptions(argc, argv)) {
    exit (1);
  }

  if (readTopology (&network)) {
    RS_PRINT_ERROR ("readTopology failed\n");
    exit (1);
  }

  printOfferedLoad ();

}

/********************************************************************************/
static int
getOptions (int argc, char **argv)
{
  int c;
  extern char *optarg;
  extern int optind;
  int errflg = 0;
  
   while ((c = getopt(argc, argv, "a:l:b:")) != EOF)
   switch (c) {
   case 'a':
     avgrate = strtod (optarg, (char **) NULL);
     break;
   case 'l':
     avglen = strtod (optarg, (char **) NULL);
     break;
   case 'b':
     avgbw = strtod (optarg, (char **) NULL);
     break;     
   case '?':
     errflg++;
   }
   if (errflg || (optind != 7)) {
     fprintf (stderr, "usage:\t%s\t-a <avg arrival rate>\n\t\t\t-l <avg duration>\n\t\t\t-b <avg bandwidth>\n", "loadcalc");
     return (-1);
   }

   return (0);
     
}
/********************************************************************************/
static void
printOfferedLoad (void)
{
  double    offeredload;
  double    avghops, avglinkcap, totalcap;
  double    numnodes, numlinks;

  avghops = getAvgHopcount (network);
  avglinkcap = getAvgLinkCapacity (network);
  totalcap = getTotalLinkCapacity (network);
  numnodes = (double)getNumNodes (network);
  numlinks = (double)getNumLinks (network);

  offeredload = avgrate * avglen * avgbw / avglinkcap;
  offeredload *= (numnodes * avghops / numlinks);

  printf ("Total offered load:\t%.4f\n", offeredload);
  printf ("Total link capacity):\t%.4f\n", totalcap);
  printf ("Avg link capacity):\t%.4f\n", avglinkcap);
  
}
