/* $Id: topology.h,v 3.2 1999/01/08 05:39:27 ashaikh Exp $ */

#ifndef _topology_h_
#define _topology_h_

#include "topologydefs.h"
#include "rsstatlist.h"
#include "rstrace.h"
#include "general.h"

#define MONFILENAME	    "rsmonitor.dat"
#define MAX_MONFILE_LINE    128
#define	DEFAULT_CONFIDENCE_LEVEL    99

/* Declarations for network graph representation */
/* Default metric values */

/* fixed link parameters */
#define DEFAULT_LINK_CAPACITY		1.0	/* Mb/sec   */
#define DEFAULT_LINK_ADMINWEIGHT	0.0
#define DEFAULT_LINK_PROPDELAY		0.0
#define DEFAULT_LINK_MAXCALLS		INT_MAX	/* calls per node */

#define DEFAULT_LINK_HOPCOUNT		1
#define DEFAULT_LINK_LOSSRATIO		0
#define DEFAULT_LINK_COST		0

/********************************************************************************/
int
initNetwork (topology_t **, topologytype_t);

int
createKaryNcube (int, int, topology_t **);
int
createFileTopology (char *, topology_t **);
int
readTopology (topology_t **);
int
createFullTopology (int, topology_t **);
int
createRandTopology (randgraph_t, int, double, double, double,
		    boolean, int, topology_t **);

int
splitNetCapacity (topology_t *, double, topology_t **, topology_t **);

link_t *
genLink (int, int, int);
int
genLinkVariables (link_t *);
int
initLinkUtilStat (link_t *, double);

int
areConnected (int, int, topology_t *);
link_t *
getLinkRef (int, int, topology_t *);

int
initLinkMonitor (topology_t *);

int
getDiameter (topology_t *);
int
getMinHops (int, int, topology_t *);
int
getNumLinks (topology_t *);
int
getNumNodes (topology_t *);
double
getAvgHopcount (topology_t *);
double
getAvgDegree (topology_t *);


int
setLinkUpdateTime (int, int, double, topology_t *);
int
updateLinks (topology_t *);
int
updateSingleLink (int, int, topology_t *);

int
setLinkUtilUpdateTime (int, int, double, topology_t *);

int
setBlockingPolicy (blockingpolicy_t, topology_t *);
void
printBlockingPolicy (topology_t *);

int
initLinkMatrix (topology_t *);
int
clearLinkMatrix (topology_t *);
void
printLinkMatrix (topology_t *);

void
printCallsPerLinkStats (topology_t *);
void
printDualCallsPerLinkStats (topology_t *, topology_t *);
void
printLinkUtilStats (topology_t *);
void
printDualLinkUtilStats (topology_t *, topology_t *);

int
initDistTable (topology_t *);
int
clearDistTable (topology_t *);
int
destroyDistTable (topology_t *);
int
calcDistTable (topology_t *);
int
parseDistTable (topology_t *);

void
printTopFunction (topology_t *);
void
printTopologyStats (topology_t *);

void
calcLinkCapacityStats (topology_t *);
double
getAvgLinkCapacity (topology_t *);
double
getMaxLinkCapacity (topology_t *);
double
getMinLinkCapacity(topology_t *);
double
getTotalLinkCapacity(topology_t *);

#if RS_TRACE_LEVEL >= TRACE_DETAILS
void printCapacityTable (topology_t *);
#endif

int dumpTopologyFile (char *, topology_t *);

#if RS_TRACE_LEVEL >= TRACE_DETAILS
void printDistTable (topology_t *);
#endif

#endif
