/* k Shortest Paths in O(E*log V + L*k*log k) time (L is path length)
   Implemented by Jon Graehl (jongraehl@earthling.net)
   Following David Eppstein's "Finding the k Shortest Paths" March 31, 1997 draft
   (http://www.ics.uci.edu/~eppstein/
    http://www.ics.uci.edu/~eppstein/pubs/p-kpath.ps)
   */

/* Testfile rewritten to make it an object file for us in routesim.
   Ganga - 09/03/2000
*/


#include "graph.h"
#include <strstream.h>
#include <iostream.h>
#include <fstream.h>

int** getAllRoutes(const char* topology_file, int source, int dest, int k)
{

  int** allroutes;
  ifstream infile;

  int kindex = 0;
  int nodeindex = 0;
  int loop_flag = 0;

  infile.open(topology_file);
  assert(infile);

  Graph graph;
  
  infile >> graph;

  infile.close();

  /* check for when source = dest in the call
   * Ganga 03/10/2000
   */

  if(source==dest) {
    allroutes = new int*[2];
    allroutes[0] = new int[3];
    allroutes[1] = new int[1];
    allroutes[0][0] = allroutes[0][1] = source;
    allroutes[0][2] = allroutes[1][0] = -1;
    return allroutes;
  }
      

  
  allroutes = new int*[k+2];
  List<List<GraphArc *> > *paths = bestPaths(graph, source, dest, k);
  for ( ListIter<List<GraphArc *> > pathIter(*paths) ; pathIter ; ++pathIter ) {
    float pathWeight = 0;
    GraphArc *a;

    allroutes[kindex] = new int[20];
    allroutes[kindex][nodeindex++] = source;
    for ( ListIter<GraphArc *> arcIter(pathIter.data()) ; arcIter ; ++arcIter ) {
      a = arcIter.data();
      allroutes[kindex][nodeindex++] = a->dest;
    }
    allroutes[kindex][nodeindex] = -1;

    //Check for loops in path.
    //If yes discard the path.

    for(int lc1 = 0; lc1 < nodeindex; lc1++) {      
      for (int lc2 = lc1+1; lc2 < nodeindex; lc2++) {
	if (allroutes[kindex][lc1]==allroutes[kindex][lc2]) {
	  loop_flag = 1;
	  break;
	}
      }
      if(loop_flag)
	break;
    }
    
    if(loop_flag==0)
      kindex++;
    nodeindex = 0;
    loop_flag = 0;
  }
  allroutes[kindex] = new int[1];
  allroutes[kindex][nodeindex] = -1;
  return allroutes;
}
