/* k Shortest Paths in O(E*log V + L*k*log k) time (L is path length)
   Implemented by Jon Graehl (jongraehl@earthling.net)
   Following David Eppstein's "Finding the k Shortest Paths" March 31, 1997 draft
   (http://www.ics.uci.edu/~eppstein/
    http://www.ics.uci.edu/~eppstein/pubs/p-kpath.ps)
   */


#include "graph.h"
#include <strstream.h>

int** putBestPaths(Graph graph, int source, int dest, int k)
{

  int** allroutes;
  
  int kindex = 0;
  int nodeindex = 0;

  allroutes = new int*[k+2];
  List<List<GraphArc *> > *paths = bestPaths(graph, source, dest, k);
  for ( ListIter<List<GraphArc *> > pathIter(*paths) ; pathIter ; ++pathIter ) {
    float pathWeight = 0;
    GraphArc *a;

    allroutes[kindex] = new int[20];
    for ( ListIter<GraphArc *> arcIter(pathIter.data()) ; arcIter ; ++arcIter ) {
      a = arcIter.data();
      allroutes[kindex][nodeindex++] = a->dest;
    }
    allroutes[kindex++][nodeindex] = -1;
    nodeindex = 0;
  }
  allroutes[kindex] = new int[1];
  allroutes[kindex][nodeindex] = -1;
  return allroutes;
}







int main(int argc, char *argv[])
{
  if ( argc != 4) {
    cout << "Test of k best paths algorithm.  Supply three integer arguments (source state number, destination state number, number of paths) and a graph to stdin.  See readme.txt, sample.graph, or ktest.cc for the graph format.\n";
    return 0;
  }
  int k;
  int source, dest;
  istrstream sstr(argv[1]);
  istrstream dstr(argv[2]);
  istrstream kstr(argv[3]);
  
  if ( !((sstr >> source) && (dstr >> dest) && (kstr >> k)) ) {
    cerr << "Bad argument (should be integer) - aborting.\n";
    return -1;
  }  Graph graph;
  cin >> graph;
  //  cout << "Outside cin graph";
  // I need to overload the = operator for Graph !!!

  //  Graph g2 = graph;
  
  
  int **allroutes;
  int kindex, nodeindex;

  //  for(int j = 0; j<2; j++)
  //    {
  //      g2 = graph;

  //  cout << "ktest2 before";
  //  fflush(NULL);

  allroutes = putBestPaths(graph, source, dest, k);
  kindex = 0;
  nodeindex = 0;
  //  cout << " I am here";
  //  fflush(NULL);
  
  while (allroutes[kindex][0] != -1)
    {
      while(allroutes[kindex][nodeindex] != -1)
	{
	  //	      cout << "kindex :" << kindex << " nodeindex :" << nodeindex << "-";
	  cout << allroutes[kindex][nodeindex++] << " ";
	}
      cout << "\n";
      kindex ++;
      nodeindex = 0;
    }  
  // Close the loop bracket here.
  //CUT FROM HERE
  
  //  cout << " Exiting program";
  //  cout << graph;
  return 0;
}








