/* $Id: topfile.c,v 1.9 1999/01/08 05:25:07 ashaikh Exp $ */

#include "topfile.h"
#include "topology.h"
#include <stdlib.h>
#include <string.h>
#include <errno.h>


extern	double	refreshinterval;

static int parseTopologyFile (FILE *topfile, topology_t **network);
/********************************************************************************/
int
genFileTopology (char *infile, topology_t **network)
{
  FILE		*topfile;


  topfile = fopen (infile, "r");
  if (topfile == NULL) {
#if RS_TRACE_LEVEL >= TRACE_ERRORS
    fprintf (stderr, "Error opening %s for reading: %s\n",
	     infile, strerror(errno));
#endif
    
    return (-1);
  }

  return (parseTopologyFile (topfile, network));
}
/********************************************************************************/
int
readStdinTopology (topology_t **network)
{
  return (parseTopologyFile (stdin, network));
}
/********************************************************************************/
static
int parseTopologyFile (FILE *topfile, topology_t **network)
{
  char		*line, *result;
  link_t	*newlink;
  int		scanresult;
  topology_t	*net;
  int		from, to, maxcalls;
  double	capacity, propdelay, adminweight;
  
  if (initNetwork(&net, TOP_FILE)) {
    RS_PRINT_ERROR ("parseTopologyFile:  initNetwork failed\n");
    return (-1);
  }

  line = (char *) malloc (MAX_TOPFILE_LINE);
  /* read number of nodes */
  result = fgets (line, MAX_TOPFILE_LINE, topfile);
  scanresult = sscanf (line, "%d \n", &(net->numnodes));
  if (scanresult != 1) {
#if RS_TRACE_LEVEL >= TRACE_SOFT_ERRORS
    fprintf (stderr, "Error parsing numnodes from file %s\n", topfile);
#endif
    return (-1);
  }

  if (initLinkMatrix (net)) {
    RS_PRINT_SOFT_ERROR ("Error in initMatrix\n");
    return (-1);
  }

  /* now read link info */
  result = fgets (line, MAX_TOPFILE_LINE, topfile);

  while (result != (char *) 0) {

    scanresult = sscanf (line, "%d %d %lf %lf %lf %d\n",
			 &from,
			 &to,
			 &capacity,
			 &propdelay,
			 &adminweight,
			 &maxcalls);

    /* perform some checks on what was scanned */
    if (scanresult < 6) {
      RS_PRINT_ERROR("Error parsing link\n");
      return (-1);
    }

    if ( (from < 0) ||
	 (from > net->numnodes - 1) ||
	 (to < 0) ||
	 (to > net->numnodes - 1) ){
      RS_PRINT_ERROR ("Link endpoints out of range\n");
      return (-1);
    }

    /* allocate link structure and fill it in with default values */
    newlink = genLink (from, to, 0);
    if (newlink == 0) {
      RS_PRINT_ERROR ("parseTopologyFile: link allocation failed\n");
      return (-1);
    }    

    /* check for non-default values */
    if (capacity != -1.0)
      newlink->capacity = capacity;
    if (propdelay != -1.0)
      newlink->propdelay = propdelay;
    if (adminweight != -1.0)
      newlink->adminweight = adminweight;
    if (maxcalls != -1)
      newlink->maxcalls = maxcalls;

    /* fill in the link variables */
    if (genLinkVariables (newlink)) {
      RS_PRINT_ERROR ("parseTopologyFile: genLinkVariables returned error\n");
      return (-1);
    }

    /* add link into the matrix */
    net->links[newlink->from][newlink->to] = newlink;
    (net->numlinks)++;

    /* read the next line */
    result = fgets (line, MAX_TOPFILE_LINE, topfile);
  }
  fclose (topfile);
  free (line);

  if (initDistTable(net)) {
    RS_PRINT_ERROR ("parseTopologyFile: Error in initDistTable()\n");
    return (-1);
  }

  if (calcDistTable(net)) {
    RS_PRINT_ERROR ("parseTopologyFile: Error in calcDistTable()\n");
    return (-1);
  }

  if (parseDistTable (net)) {
    RS_PRINT_ERROR ("parseTopologyFile:  Error in parseDistTable()\n");
    return (-1);
  }

  net->info = (void *) 0;

  calcLinkCapacityStats(net);

  *network = net;
  return (0);
}

/********************************************************************************/
