/* $Id: topfull.c,v 1.4 1998/12/07 18:54:01 ashaikh Exp $ */

#include "topfull.h"
#include "topology.h"
#include <stdlib.h>

/********************************************************************************/
int
genFullTopology (int size, topology_t **network)
{
  int		i, j, neighbors;
  topology_t	*net;
  link_t	*link;
  
  if (size == 0) {
    RS_PRINT_ERROR ("genFullTopology: Topology size not initialized\n");
    return (-1);
  }

  if (initNetwork(&net, TOP_FULL)) {
    RS_PRINT_ERROR ("genFullTopology:  initNetwork failed\n");
    return (-1);
  }

  net->numnodes = size;

  if (initLinkMatrix (net)) {
    RS_PRINT_SOFT_ERROR ("genFullTopology: Error in initMatrix\n");
    return (-1);
  }

  /* make all the links */
  for (i = 0; i < net->numnodes; i++) {
    neighbors = 0;
    for (j = 0; j < net->numnodes; j++) {
      if (j == i) {
	continue;
      }
      else {
	link = genLink(i, j, 0);
	if (link == (link_t *) 0) {
	  RS_PRINT_ERROR ("genFullTopology: Link alloc failed\n");
	  return (-1);
	}
	if (genLinkVariables (link)) {
	  RS_PRINT_ERROR ("genFullTopology: genLinkVariables returned error\n");
	  return (-1);
	}
	net->links[i][j] = link;      
	net->numlinks++;
	neighbors++;
      }
    }
    updateSimpleRatioStat (neighbors, &(net->degree));
  }

  /* just a double check */
  if (net->numlinks != net->numnodes * (net->numnodes - 1)) {
    RS_PRINT_ALWAYS ("genFullTopology: Mismatch in numlinks calculation\n");
    return (-1);
  }
  if (net->degree.currentrate != (net->numnodes - 1.0)) {
    RS_PRINT_ERROR ("genFullTopology: Mismatch in degree calculation\n");
    return (-1);
  }

  /* initialize the distance table */
  /* for a fully connected topology, allocating this matrix is kind of a waste */
  /* but we do it anyway for consistency with other topologies. */

  if (initDistTable(net)) {
    RS_PRINT_ERROR ("genFullTopology: Error in initDistTable()\n");
    return (-1);
  }

  for (i = 0; i < net->numnodes; i++) {
    for (j = 0; j < net->numnodes; j++) {
      if (j == i) {
	net->hoptable[i][j] = 0;
      }
      else {
	net->hoptable[i][j] = 1;
	updateSimpleRatioStat (net->hoptable[i][j], &(net->hopcount));	
      }
    }
  }

  /* some checks */
  if (net->hopcount.currentrate != 1.0) {
    RS_PRINT_ALWAYS ("Mismatch in avghops calculation\n");
    return (-1);
  }          

  if (net->hopcount.max != 1) {
    RS_PRINT_ALWAYS ("Mismatch in diameter calculation\n");
    return (-1);
  }
  net->diameter = 1;


  net->info = (void *) 0;

  calcLinkCapacityStats(net);

  *network = net;
  
  return (0);
}
/********************************************************************************/
